
package mainpack;


import datapack.ThreadData;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.logging.Level;
import java.util.logging.Logger;
import threadspack.AvailableInterfaces;
import threadspack.MonitorDataBuilder;

/**The class that contains the main method. */
public class Main
{  
    /**Map that stores all opened threads by the program. */
    public static volatile Map<String, ThreadData> threadsTable = new HashMap();  
    /**The properties file that contains the initial setup info. */
    public static Properties prop = new Properties();
    /**List that contains all the states of the Markov Chain. */
    public static List<Integer> markovList = new LinkedList();
    /**AvailableInterfaces Object that is the thread to monitor the available interfaces. */
    public static AvailableInterfaces ai = new AvailableInterfaces();
    /**Starting time of running the program. */
    public static final long start_time = System.currentTimeMillis();
    /**MonitorDataBuilder object that is the thread to send periodically all the collected data. */
    private static MonitorDataBuilder dp = new MonitorDataBuilder();    
    
    /**Main method that loads the properties from the property file,
     * builds the markovList based on the properties loaded,
     * and starts the threads for the available interfaces thread and the data printer thread
     * This method also contains an anonymous thread that is added a shutdown hook.
     * @param args Argument given by the user when running the program that contains the path of the property file. */
    public static void main(String[] args)
    {       
        FileInputStream in = null;
        
        if (args.length == 0)
        {
            System.out.println("Not enough arguments! The program will now exit...");
            System.exit(0);
        }
       
        try {
            in = new FileInputStream(args[0]);
            prop.load(in);
        }catch (FileNotFoundException ex)
        {

            Logger lgr = Logger.getLogger(Main.class.getName());
            lgr.log(Level.SEVERE, ex.getMessage(), ex);

        } catch (IOException ex)
        {

            Logger lgr = Logger.getLogger(Main.class.getName());
            lgr.log(Level.SEVERE, ex.getMessage(), ex);

        } finally
        {
            try
            {
                if (in != null)
                {
                    in.close();                    
                }
            } catch (IOException ex)
            {
                Logger lgr = Logger.getLogger(Main.class.getName());
                lgr.log(Level.SEVERE, ex.getMessage(), ex);
            }            
        }
        
        int s = Integer.parseInt(prop.getProperty("sleeptime")), temps;
        int r, maxr = Integer.parseInt(prop.getProperty("repetitions"));
        int chrate = Integer.parseInt(prop.getProperty("changerate").replace("%", ""));
        int states = Integer.parseInt(prop.getProperty("states"));
        int maxs = (states-1) * (s*chrate/100)  + s;        
        
        System.out.println(System.currentTimeMillis()-start_time+"\t[main] INFO MonitorStarter - Monitor initialized with the following properties:");
        System.out.println(System.currentTimeMillis()-start_time+"\t[main] INFO MonitorStarter - Starting SleepTime: "+s);
        System.out.println(System.currentTimeMillis()-start_time+"\t[main] INFO MonitorStarter - Unsuccessful Repetitions: "+maxr);
        System.out.println(System.currentTimeMillis()-start_time+"\t[main] INFO MonitorStarter - SleepTime ChangeRate: "+chrate+"%");
        System.out.println(System.currentTimeMillis()-start_time+"\t[main] INFO MonitorStarter - States: "+states);                
 
        //markov list creation
        temps = s;
        while ( temps <= maxs ) 
        { 
            //node overflow depended on the repetitions
            for ( r = 0; r < maxr; r++)
            {
                markovList.add(temps);
            }
            temps += (s*chrate/100);
        }
        
        ai.startThread();   //starts available interfaces monitor thread
        ThreadData ai_thread = new ThreadData(ai, true); 
        Main.threadsTable.put("MasterThread", ai_thread);
        
        System.out.println(System.currentTimeMillis()-start_time+"\t["+ai.getName()+"] INFO MonitorStarter - AvailableInterfacesMonitor started"); 
        
        dp.startThread();   //starts data printer thread
        ThreadData dp_thread = new ThreadData(dp, true); 
        Main.threadsTable.put("DataPrinter", dp_thread);     

        //anonymous thread added shutdown hook
        Runtime.getRuntime().addShutdownHook(new Thread()
        {
            @Override
            public void run()
            {
                System.out.println();
                runWhenShuttingDown();
            }
        });
             
    }
    
    /**Method that is called by the thread that has shutdown hook.Sends 
     * to the MonitorDataBuilder thread and to the AvailableInterfaces thread a termination signal
     * and then it waits for them to stop. */
    private static void runWhenShuttingDown()
    {
        threadsTable.get("DataPrinter").setState(false);   //send to MasterThread termination signal
        
        try
        {
            dp.join();      
            
        } catch (InterruptedException ex)
        {
            Logger.getLogger(Main.class.getName()).log(Level.SEVERE, null, ex);
            System.exit(1);
        }        
        
        threadsTable.get("MasterThread").setState(false);
        
        try
        {
            ai.join();
            
            System.out.println(System.currentTimeMillis()-start_time+"\t["+ai.getName()
                               +"] INFO MonitorStarter - AvailableInterfacesMonitor terminated");
            
        } catch (InterruptedException ex)
        {
            Logger.getLogger(Main.class.getName()).log(Level.SEVERE, null, ex);
            System.exit(1);
        }          
    }    
}