
package dbpack;

import com.mysql.jdbc.Connection;
import context.AccessPointData;
import context.InterfaceData;
import context.MonitorData;
import context.TerminalData;
import context.WirelessData;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;
import main.Main;


/**Singleton class that contains the necessary actions for connecting and disconnecting to the db,
 * creating and deleting tables, inserting, updating and removing data from the database, and also
 * creates and manages the middle memory. */
public class DatabaseActions extends Thread
{
    /**The private instance of this singleton class. */
    private static DatabaseActions instance = null;
    
    /**URL for the connection to the database. */
    private final String url = "jdbc:mysql://"+Main.props.getProperty("db.ip")
                                                     +":"+Main.props.getProperty("db.port")
                                                     +"/"+Main.props.getProperty("db.name");
    /**Username for the connection to the database. */
    private final String user = Main.props.getProperty("db.user");
    /**Password for the connection to the database. */
    private final String password = Main.props.getProperty("db.passwd");
    
    /**Connection variable for connecting to the database. */
    private Connection con;
    /**Statement variable for making statements to the database. */
    private Statement st = null;
    /**PreparedStatement variable for making prepared statements to the database. */
    private PreparedStatement pst = null;
    /**ResultSet variable for taking the results from a query to the database. */
    private ResultSet rs = null;
    
    /**Map that stores the current bit rate from all devices' local loopbacks. */
    private volatile Map<String, String> localTable = new HashMap();
    /**Map that stores the necessary data from all devices' disconnected interfaces. */
    private volatile Map<String, String> discIfTable = new HashMap();
    /**Map that stores the necessary data from all devices' connected interfaces. */
    private volatile Map<String, String[]> conIfTable = new HashMap();
    /**Map that stores the necessary data from all devices' disconnected wireless interfaces. */
    private volatile Map<String, String[]> discWifTable = new HashMap();
    /**Map that stores the necessary data from all devices' connected wireless interfaces. */
    private volatile Map<String, String[]> conWifTable = new HashMap();
    /**Map that stores the necessary data from all discovered access points. */
    private volatile Map<String, String[]> apTable = new HashMap();
    /**Map that stores the signal level from all devices' discovered access points. */
    private volatile Map<String, String> devapTable = new HashMap();
    /**Map that stores the latest update times from all devices. */
    private volatile Map<String, Long> devtimesTable = new HashMap();
    /**Map that stores the latest update times from all terminals. */
    private volatile Map<String, Long> termtimesTable = new HashMap();
    /**Map that stores the necessary data from all terminals. */
    private volatile Map<String, String[]> termDataTable = new HashMap();

    /**Private constructor for the singleton class that makes the connection to the db
     * and then checks and prints the version of mysql, deletes previously created tables
     * and creates the necessary tables in the database. */
    private DatabaseActions()
    {              
        try
        {
            con = (Connection) DriverManager.getConnection(url, user, password);
        } catch (SQLException ex)
        {
            Logger.getLogger(DatabaseActions.class.getName()).log(Level.SEVERE, null, ex);
        }
        
        this.checkVersion();
        this.deleteTables();
        this.createTables();
        this.showTables();         
    }
    
    /**Creates the unique object of this class if not created.
     * @return The unique instance to this singleton class*/
    public static synchronized DatabaseActions getInstance()
    {
        if (instance == null)
        {
            instance = new DatabaseActions();
        }
        return instance;
    }
    
    @Override
    public Object clone() throws CloneNotSupportedException
    {
        throw new CloneNotSupportedException();
    }

    /**Checks the version of the database and prints it to the terminal. */
    private void checkVersion()
    {
        try
        {
            st = con.createStatement();
            rs = st.executeQuery("SELECT VERSION()");

            if (rs.next())
            {
                System.out.println(rs.getString(1));
            }

        } catch (SQLException ex)
        {
            Logger lgr = Logger.getLogger(DatabaseActions.class.getName());
            lgr.log(Level.SEVERE, ex.getMessage(), ex);

        } finally
        {
            try
            {
                if (rs != null)
                {
                    rs.close();
                }
                if (st != null)
                {
                    st.close();
                }

            } catch (SQLException ex)
            {
                Logger lgr = Logger.getLogger(DatabaseActions.class.getName());
                lgr.log(Level.WARNING, ex.getMessage(), ex);
            }
        }

    }
   
    /**Creates the necessary tables to the database for the program. */
    private void createTables()
    {
        try
        {                       
            pst = con.prepareStatement("CREATE TABLE IF NOT EXISTS InterfaceData("
                                       + "DeviceName VARCHAR(20) NOT NULL, "
                                       + "IfName VARCHAR(20) NOT NULL, "
                                       + "MacAddr VARCHAR(20) NULL, "
                                       + "IpAddr VARCHAR(20) NULL, "
                                       + "SubMask VarChar(20) NULL, "
                                       + "NetAddr VARCHAR(20) NULL, "
                                       + "BcastAddr VARCHAR(20) NULL, "
                                       + "DefGateway VARCHAR(20) NULL, "
                                       + "MaxBitRate VARCHAR(30) NULL, "
                                       + "CurBitRate VARCHAR(30) NULL, "
                                       + "CurUsedRange VARCHAR(30) NULL, "
                                       + "PER VARCHAR(30) NULL, "
                                       + "timestamp TIMESTAMP, "
                                       + "PRIMARY KEY (DeviceName, IfName) "
                                       + ") ENGINE=InnoDB");
            pst.executeUpdate();
            
            pst = con.prepareStatement("CREATE TABLE IF NOT EXISTS AccessPointsData("                    
                                       + "MacAddr VARCHAR(20) NOT NULL, "
                                       + "Essid VARCHAR(35) NOT NULL, "
                                       + "Channel VARCHAR(15) NOT NULL, "
                                       + "Mode VARCHAR(15) NOT NULL, "
                                       + "timestamp TIMESTAMP, "
                                       + "PRIMARY KEY (MacAddr) "
                                       + ") ENGINE=InnoDB");
            pst.executeUpdate();   
            
            pst = con.prepareStatement("CREATE TABLE IF NOT EXISTS DeviceAccessPoints("   
                                       + "DeviceName VARCHAR(20) NOT NULL, "                    
                                       + "ApMacAddr VARCHAR(20) NOT NULL, "
                                       + "SignalLevel VARCHAR(15) NOT NULL, "                    
                                       + "timestamp TIMESTAMP, "
                                       + "PRIMARY KEY (DeviceName, ApMacAddr), "
                                       + "FOREIGN KEY(ApMacAddr) REFERENCES AccessPointsData(MacAddr)"                    
                                       + ") ENGINE=InnoDB");
            pst.executeUpdate();             
            
            pst = con.prepareStatement("CREATE TABLE IF NOT EXISTS WirelessData("
                                       + "DeviceName VARCHAR(20) NOT NULL, "
                                       + "IfName VARCHAR(20) NOT NULL, "
                                       + "MacAddr VARCHAR(20) NOT NULL, "
                                       + "IpAddr VARCHAR(20) NULL, "
                                       + "SubMask VarChar(20) NULL, "                    
                                       + "NetAddr VARCHAR(20) NULL, "
                                       + "BcastAddr VARCHAR(20) NULL, "
                                       + "DefGateway VARCHAR(20) NULL, "
                                       + "MaxBitRate VARCHAR(30) NULL, "
                                       + "CurBitRate VARCHAR(30) NULL, "
                                       + "CurUsedRange VARCHAR(30) NULL, "
                                       + "PER VARCHAR(30) NULL, "
                                       + "ApMacAddr VARCHAR(20) NULL, "
                                       + "TPower VARCHAR(15) NULL, "
                                       + "LinkQuality VARCHAR(15) NULL, "
                                       + "SignalLevel VARCHAR(15) NULL, "
                                       + "NoiseLevel VARCHAR(15) NULL, "
                                       + "DiscPacks VARCHAR(20) NULL, "
                                       + "timestamp TIMESTAMP, "
                                       + "PRIMARY KEY (DeviceName, IfName) "                   
                                       + ") ENGINE=InnoDB");
            pst.executeUpdate(); 
            
            pst = con.prepareStatement("CREATE TABLE IF NOT EXISTS TerminalData("
                                       + "IMEI VARCHAR(20) NOT NULL, "
                                       + "Model VARCHAR(20) NOT NULL, "
                                       + "Manufacturer VARCHAR(20) NOT NULL, "
                                       + "Version VARCHAR(10) NULL, "
                                       + "Latitude VarChar(20) NULL, "                    
                                       + "Longitude VARCHAR(20) NULL, "
                                       + "BatteryLevel VARCHAR(20) NULL, "
                                       + "Charging VARCHAR(20) NULL, "
                                       + "timestamp TIMESTAMP, "
                                       + "PRIMARY KEY (IMEI) "                   
                                       + ") ENGINE=InnoDB");
            pst.executeUpdate();              
        } catch (SQLException ex)
        {
            Logger.getLogger(DatabaseActions.class.getName()).log(Level.SEVERE, null, ex);
        } finally
        {
            try
            {
                if (pst != null)
                {
                    pst.close();
                }          
                
            } catch (SQLException ex)
            {
                Logger.getLogger(DatabaseActions.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
    }

    /**Gets the data sent by the terminal and do the necessary actions to store them to the database
     * and the middle memory, and refreshes the gui if necessary. 
     * @param device The name of the terminal that sent the data
     * @param TD TerminalData object with all the data sent by the terminal*/
    public synchronized void insertData(String device, TerminalData TD)
    {
        boolean existed = false;

        try
        {
            if (this.termtimesTable.containsKey(device))
            {
                existed = true;
            }

            //stores the time that a terminal sent its data
            this.termtimesTable.put(device, System.currentTimeMillis());

            if (existed)
            {
                if (!this.termDataTable.get(device)[0].equals(String.valueOf(TD.getLatitude())))
                {
                    //updates connected interface table
                    this.termDataTable.get(device)[0] = String.valueOf(TD.getLatitude());

                    //updates the database
                    this.updateTerminalData("Latitude", String.valueOf(TD.getLatitude()), device);  

                    //updates the gui
                    if(device.equals(Main.gui.getActiveDevice()))
                    {
                        Main.gui.refreshTerminal();
                    }                   
                }

                if (!this.termDataTable.get(device)[1].equals(String.valueOf(TD.getLongitude())))
                {
                    //updates connected interface table
                    this.termDataTable.get(device)[1] = String.valueOf(TD.getLongitude());

                    //updates the database
                    this.updateTerminalData("Longitude", String.valueOf(TD.getLongitude()), device);  

                    //updates the gui
                    if(device.equals(Main.gui.getActiveDevice()))
                    {
                        Main.gui.refreshTerminal();
                    }                      
                }

                if (!this.termDataTable.get(device)[2].equals(String.valueOf(TD.getBatteryLevel())))
                {
                    System.out.println(this.termDataTable.get(device)[2]+" == "+String.valueOf(TD.getBatteryLevel()));
                    
                    //updates connected interface table
                    this.termDataTable.get(device)[2] = String.valueOf(TD.getBatteryLevel());

                    //updates the database
                    this.updateTerminalData("BatteryLevel", String.valueOf(TD.getBatteryLevel()), device);  

                    //updates the gui
                    if(device.equals(Main.gui.getActiveDevice()))
                    {
                        Main.gui.refreshTerminal();
                    }                     
                }

                if (!this.termDataTable.get(device)[3].equals(TD.isCharging() ? "Charging" : "Not Charging"))
                {
                    //updates connected interface table
                    this.termDataTable.get(device)[3] = (TD.isCharging() ? "Charging" : "Not Charging");

                    //updates the database
                    this.updateTerminalData("Charging", (TD.isCharging() ? "Charging" : "Not Charging"), device);  

                    //updates the gui
                    if(device.equals(Main.gui.getActiveDevice()))
                    {
                        Main.gui.refreshTerminal();
                    }                     
                }
            }
            else
            {                
                pst = con.prepareStatement("INSERT INTO TerminalData(IMEI, Model, Manufacturer, Version, Latitude, "
                        + "Longitude, BatteryLevel, Charging) VALUES(?,?,?,?,?,?,?,?)");

                pst.setString(1, device);
                pst.setString(2, TD.getModel());
                pst.setString(3, TD.getManufacturer());
                pst.setString(4, TD.getVersion());
                pst.setString(5, String.valueOf(TD.getLatitude()));
                pst.setString(6, String.valueOf(TD.getLongitude()));
                pst.setString(7, String.valueOf(TD.getBatteryLevel()));
                pst.setString(8, (TD.isCharging() ? "Charging" : "Not Charging"));

                pst.executeUpdate();
                pst.close();
                
                String[] temparray = new String[4];                        
                //initiates the values in the table
                temparray[0] = String.valueOf(TD.getLatitude());
                temparray[1] = String.valueOf(TD.getLongitude()); 
                temparray[2] = String.valueOf(TD.getBatteryLevel());
                temparray[3] = (TD.isCharging() ? "Charging" : "Not Charging");

                //adds it in the connected table
                this.termDataTable.put(device, temparray);                 
                
                Main.gui.addTerminal(device);

            }
        } catch (SQLException ex)
        {
            Logger lgr = Logger.getLogger(DatabaseActions.class.getName());
            lgr.log(Level.SEVERE, ex.getMessage(), ex);

        } finally
        {
            try
            {
                if (pst != null)
                {
                    pst.close();
                }

            } catch (SQLException ex)
            {
                Logger lgr = Logger.getLogger(DatabaseActions.class.getName());
                lgr.log(Level.SEVERE, ex.getMessage(), ex);
            }
        }      
    }    
    
    /**Gets the data sent by the device and do the necessary actions to store them to the database
     * and the middle memory, and refreshes the gui if necessary. 
     * @param device The name of the device that sent the data
     * @param MD MonitorData object with all the data sent by the device*/
    public synchronized void insertData(String device, MonitorData MD)
    {
        int i;
        boolean newarrival = false, existed = true;
        InterfaceData tempif;
        WirelessData tempwif;
        AccessPointData tempap;
        
        //stores the time that a device sent its data
        this.devtimesTable.put(device, System.currentTimeMillis());
        //in this list will be added every interface that will be found, to see at the end which intrfaces where disconnected
        List<String> ifexist = new ArrayList();

        try
        {
            for (i = 0; i < MD.getInterfaceList().size(); i++)
            {
                tempif = MD.getInterfaceList().get(i);
                
                if (tempif.getName().equals("lo")) //it is the local loopback
                {
                    if (this.localTable.containsKey(device)) //already exists in the local loopback table
                    {
                        //if the value CurBitRate changed
                        if(!this.localTable.get(device).equals(tempif.getCurrent_bitrate()))
                        {
                            //updates local loopback table
                            this.localTable.put(device, tempif.getCurrent_bitrate());
                            
                            //updates the database
                            this.updateData("InterfaceData", "CurBitRate", tempif.getCurrent_bitrate(), device, tempif.getName());   
                            
                            //updates the gui
                            if(device.equals(Main.gui.getActiveDevice()) && Main.gui.getActiveIf().equals(tempif.getName()))
                            {
                                Main.gui.refresh();
                            }                           
                        }
                    }
                    else //doesnt exist
                    {
                        this.localTable.put(device, tempif.getCurrent_bitrate());
                        existed = false;
                        newarrival = true;
                        
                        Main.gui.addDevice(device);
                    }
                }
                else if (tempif.getIp_address().equals("Not-available")) //it is disconnected
                {                                       
                    if (this.discIfTable.containsKey(device + tempif.getName()) && existed) //already exists in the disconnected intefaces table
                    {
                        //if the value MacAddr changed
                        if(!this.discIfTable.get(device+tempif.getName()).equals(tempif.getMac_address()))
                        {
                            //updates disconnected interface table
                            this.discIfTable.put(device+tempif.getName(), tempif.getMac_address());
                            
                            //updates the database
                            this.updateData("InterfaceData", "MacAddr", tempif.getMac_address(), device, tempif.getName()); 
                            
                            //updates the gui
                            if(device.equals(Main.gui.getActiveDevice()) && Main.gui.getActiveIf().equals(tempif.getName()))
                            {
                                Main.gui.refresh();
                            }                             
                        }                                               
                    }
                    else if (this.conIfTable.containsKey(device + tempif.getName()) && existed) //was connected but now is disconnected
                    {                              
                        //removes it from the connected list
                        this.conIfTable.remove(device + tempif.getName());                    
                        
                        //adds it in the disconnected table
                        this.discIfTable.put(device + tempif.getName(), tempif.getMac_address());                                             
                        
                        //changes the values in the database
                        this.updateData("InterfaceData", "IpAddr", tempif.getIp_address(), device, tempif.getName());
                        this.updateData("InterfaceData", "SubMask", tempif.getSubnetmask(), device, tempif.getName());
                        this.updateData("InterfaceData", "NetAddr", tempif.getNetwork_address(), device, tempif.getName());
                        this.updateData("InterfaceData", "BcastAddr", tempif.getBroadcast_address(), device, tempif.getName());
                        this.updateData("InterfaceData", "DefGateway", tempif.getDefault_gateway(), device, tempif.getName());
                        this.updateData("InterfaceData", "MaxBitRate", tempif.getMax_bitrate(), device, tempif.getName());
                        this.updateData("InterfaceData", "CurBitRate", tempif.getCurrent_bitrate(), device, tempif.getName());
                        this.updateData("InterfaceData", "CurUsedRange", tempif.getCurrent_usedrange(), device, tempif.getName());   
                        
                        //updates the gui
                        if(device.equals(Main.gui.getActiveDevice()) && Main.gui.getActiveIf().equals(tempif.getName()))
                        {
                            Main.gui.refresh();
                        }                         
                    }
                    else //doesnt exist in any table
                    {                        
                        //adds it in the disconnected table
                        this.discIfTable.put(device + tempif.getName(), tempif.getMac_address());
                        
                        newarrival = true;                        
                    }
                }
                else //is connected
                {
                    if (this.conIfTable.containsKey(device + tempif.getName()) && existed) //already exists in the connected intefaces table
                    {
                        //if the value MacAddr changed
                        if(!this.conIfTable.get(device+tempif.getName())[0].equals(tempif.getMac_address()))
                        {
                            //updates connected interface table
                            this.conIfTable.get(device+tempif.getName())[0] = tempif.getMac_address();
                            
                            //updates the database
                            this.updateData("InterfaceData", "MacAddr", tempif.getMac_address(), device, tempif.getName());  
                            
                            //updates the gui
                            if(device.equals(Main.gui.getActiveDevice()) && Main.gui.getActiveIf().equals(tempif.getName()))
                            {
                                Main.gui.refresh();
                            }                             
                        }
                        
                        //if one of the values IpAddr or SubMask changed
                        if(!this.conIfTable.get(device+tempif.getName())[1].equals(tempif.getIp_address()) 
                           || !this.conIfTable.get(device+tempif.getName())[2].equals(tempif.getSubnetmask()))
                        {
                            //updates connected interface table
                            this.conIfTable.get(device+tempif.getName())[1] = tempif.getIp_address();
                            
                            //updates the database
                            this.updateData("InterfaceData", "IpAddr", tempif.getIp_address(), device, tempif.getName());    
                            
                            //updates connected interface table
                            this.conIfTable.get(device+tempif.getName())[2] = tempif.getSubnetmask();
                            
                            //updates the database
                            this.updateData("InterfaceData", "SubMask", tempif.getSubnetmask(), device, tempif.getName());
                            
                            //updates the network and broadcast addresses in the database
                            this.updateData("InterfaceData", "NetAddr", tempif.getNetwork_address(), device, tempif.getName());
                            this.updateData("InterfaceData", "BcastAddr", tempif.getBroadcast_address(), device, tempif.getName());  
                            
                            //updates the gui
                            if(device.equals(Main.gui.getActiveDevice()) && Main.gui.getActiveIf().equals(tempif.getName()))
                            {
                                Main.gui.refresh();
                            }                             
                        }
                        
                        //if the value DefGateway changed
                        if(!this.conIfTable.get(device+tempif.getName())[3].equals(tempif.getDefault_gateway()))
                        {
                            //updates connected interface table
                            this.conIfTable.get(device+tempif.getName())[3] = tempif.getDefault_gateway();
                            
                            //updates the database
                            this.updateData("InterfaceData", "DefGateway", tempif.getDefault_gateway(), device, tempif.getName());   
                            
                            //updates the gui
                            if(device.equals(Main.gui.getActiveDevice()) && Main.gui.getActiveIf().equals(tempif.getName()))
                            {
                                Main.gui.refresh();
                            }                             
                        }  
                        
                        //if one of the values MaxBitRate or CurBitRate changed
                        if(!this.conIfTable.get(device+tempif.getName())[4].equals(tempif.getMax_bitrate()) 
                           || !this.conIfTable.get(device+tempif.getName())[5].equals(tempif.getCurrent_bitrate()))
                        {
                            //updates connected interface table
                            this.conIfTable.get(device+tempif.getName())[4] = tempif.getMax_bitrate();
                            
                            //updates the database
                            this.updateData("InterfaceData", "MaxBitRate", tempif.getMax_bitrate(), device, tempif.getName());    
                            
                            //updates connected interface table
                            this.conIfTable.get(device+tempif.getName())[5] = tempif.getCurrent_bitrate();
                            
                            //updates the database
                            this.updateData("InterfaceData", "CurBitRate", tempif.getCurrent_bitrate(), device, tempif.getName());
                            
                            //updates the CurUsedRange in the database
                            this.updateData("InterfaceData", "CurUsedRange", tempif.getCurrent_usedrange(), device, tempif.getName()); 
                            
                            //updates the gui
                            if(device.equals(Main.gui.getActiveDevice()) && Main.gui.getActiveIf().equals(tempif.getName()))
                            {
                                Main.gui.refresh();
                            }                             
                        }    
                        
                        //if the value PER changed
                        if(!this.conIfTable.get(device+tempif.getName())[6].equals(tempif.getPacket_error_rate()))
                        {
                            //updates connected interface table
                            this.conIfTable.get(device+tempif.getName())[6] = tempif.getPacket_error_rate();
                            
                            //updates the database
                            this.updateData("InterfaceData", "PER", tempif.getPacket_error_rate(), device, tempif.getName());  
                            
                            //updates the gui
                            if(device.equals(Main.gui.getActiveDevice()) && Main.gui.getActiveIf().equals(tempif.getName()))
                            {
                                Main.gui.refresh();
                            }                             
                        }                         
                    }
                    else if (this.discIfTable.containsKey(device + tempif.getName()) && existed) //was disconnected but now is connected
                    {
                        //removes it from the disconnected list
                        this.discIfTable.remove(device + tempif.getName());
                        
                        String[] temparray = new String[7];                        
                        //initiates the values in the table
                        temparray[0] = tempif.getMac_address();
                        temparray[1] = tempif.getIp_address(); 
                        temparray[2] = tempif.getSubnetmask();
                        temparray[3] = tempif.getDefault_gateway();
                        temparray[4] = tempif.getMax_bitrate();
                        temparray[5] = tempif.getCurrent_bitrate();
                        temparray[6] = tempif.getPacket_error_rate();
                        
                        //adds it in the connected table
                        this.conIfTable.put(device + tempif.getName(), temparray);                                             
                        
                        //changes the values in the database
                        this.updateData("InterfaceData", "IpAddr", tempif.getIp_address(), device, tempif.getName());
                        this.updateData("InterfaceData", "SubMask", tempif.getSubnetmask(), device, tempif.getName());
                        this.updateData("InterfaceData", "NetAddr", tempif.getNetwork_address(), device, tempif.getName());
                        this.updateData("InterfaceData", "BcastAddr", tempif.getBroadcast_address(), device, tempif.getName());
                        this.updateData("InterfaceData", "DefGateway", tempif.getDefault_gateway(), device, tempif.getName());
                        this.updateData("InterfaceData", "MaxBitRate", tempif.getMax_bitrate(), device, tempif.getName());
                        this.updateData("InterfaceData", "CurBitRate", tempif.getCurrent_bitrate(), device, tempif.getName());
                        this.updateData("InterfaceData", "CurUsedRange", tempif.getCurrent_usedrange(), device, tempif.getName()); 
                        this.updateData("InterfaceData", "PER", tempif.getPacket_error_rate(), device, tempif.getName());  
                        
                        //updates the gui
                        if(device.equals(Main.gui.getActiveDevice()) && Main.gui.getActiveIf().equals(tempif.getName()))
                        {
                            Main.gui.refresh();
                        }                         
                    }
                    else //doesnt exist in any table
                    {
                        String[] temparray = new String[7];                        
                        //initiates the values in the table
                        temparray[0] = tempif.getMac_address();
                        temparray[1] = tempif.getIp_address(); 
                        temparray[2] = tempif.getSubnetmask();
                        temparray[3] = tempif.getDefault_gateway();
                        temparray[4] = tempif.getMax_bitrate();
                        temparray[5] = tempif.getCurrent_bitrate();
                        temparray[6] = tempif.getPacket_error_rate();
                        
                        //adds it in the connected table
                        this.conIfTable.put(device + tempif.getName(), temparray); 
                        
                        newarrival = true;                         
                    }                    
                }

                if (newarrival)
                {
                    pst = con.prepareStatement("INSERT INTO InterfaceData(DeviceName, IfName, MacAddr, IpAddr, SubMask, "
                            + "NetAddr, BcastAddr, DefGateway, MaxBitRate, CurBitRate, CurUsedRange, PER) "
                            + "VALUES(?,?,?,?,?,?,?,?,?,?,?,?)");
                    pst.setString(1, device);
                    pst.setString(2, tempif.getName());
                    pst.setString(3, tempif.getMac_address());
                    pst.setString(4, tempif.getIp_address());
                    pst.setString(5, tempif.getSubnetmask());
                    pst.setString(6, tempif.getNetwork_address());
                    pst.setString(7, tempif.getBroadcast_address());
                    pst.setString(8, tempif.getDefault_gateway());
                    pst.setString(9, tempif.getMax_bitrate());
                    pst.setString(10, tempif.getCurrent_bitrate());
                    pst.setString(11, tempif.getCurrent_usedrange());
                    pst.setString(12, tempif.getPacket_error_rate());
                    pst.executeUpdate();
                    pst.close();
                    
                    if(existed && Main.gui.getActiveDevice().equals(device))
                    {
                        Main.gui.addInterface(tempif.getName());
                    }

                    newarrival = false;
                }
                
                if (!tempif.getName().equals("lo"))
                    ifexist.add(device + tempif.getName());
            }
            
            for(Iterator<Map.Entry<String, String>> it = discIfTable.entrySet().iterator(); it.hasNext(); )
            {
                Map.Entry<String, String> entry = it.next();
                if (!ifexist.contains(entry.getKey()) && entry.getKey().startsWith(device))
                {                                       
                    this.deleteInterface(device, entry.getKey().replace(device, ""));
                    if(Main.gui.getActiveDevice().equals(device))
                    {
                        Main.gui.removeInterface(device, entry.getKey().replace(device, ""));
                    }
                    it.remove();                    
                }
            } 
            for(Iterator<Map.Entry<String, String[]>> it = conIfTable.entrySet().iterator(); it.hasNext(); )
            {
                Map.Entry<String, String[]> entry = it.next();
                if (!ifexist.contains(entry.getKey()) && entry.getKey().startsWith(device))
                {                                       
                    this.deleteInterface(device, entry.getKey().replace(device, ""));
                    if(Main.gui.getActiveDevice().equals(device))
                    {
                        Main.gui.removeInterface(device, entry.getKey().replace(device, ""));
                    }
                    it.remove();                    
                }
            }   
            ifexist.removeAll(ifexist);
                        
            for (i = 0; i < MD.getWirelessList().size(); i++)
            {
                tempwif = MD.getWirelessList().get(i);
                
                if (tempwif.getIp_address().equals("Not-available")) //it is disconnected
                {
                    if (this.discWifTable.containsKey(device + tempwif.getName()) && existed) //already exists in the disconnected intefaces table
                    {
                        //if the value MacAddr changed
                        if(!this.discWifTable.get(device+tempwif.getName())[0].equals(tempwif.getMac_address()))
                        {
                            //updates disconnected interface table
                            this.discWifTable.get(device+tempwif.getName())[0] = tempwif.getMac_address();
                            
                            //updates the database
                            this.updateData("WirelessData", "MacAddr", tempwif.getMac_address(), device, tempwif.getName());   
                            
                            //updates the gui
                            if(device.equals(Main.gui.getActiveDevice()) && Main.gui.getActiveIf().equals(tempwif.getName()))
                            {
                                Main.gui.refresh();
                            }                             
                        }
                        
                        //if one of the values MaxBitRate or CurBitRate changed
                        if(!this.discWifTable.get(device+tempwif.getName())[1].equals(tempwif.getMax_bitrate()) 
                           || !this.discWifTable.get(device+tempwif.getName())[2].equals(tempwif.getCurrent_bitrate()))
                        {
                            //updates connected interface table
                            this.discWifTable.get(device+tempwif.getName())[1] = tempwif.getMax_bitrate();
                            
                            //updates the database
                            this.updateData("WirelessData", "MaxBitRate", tempwif.getMax_bitrate(), device, tempwif.getName());    
                            
                            //updates connected interface table
                            this.discWifTable.get(device+tempwif.getName())[2] = tempwif.getCurrent_bitrate();
                            
                            //updates the database
                            this.updateData("WirelessData", "CurBitRate", tempwif.getCurrent_bitrate(), device, tempwif.getName());  
                            
                            //updates the CurUsedRange in the database
                            this.updateData("WirelessData", "CurUsedRange", tempwif.getCurrent_usedrange(), device, tempwif.getName());  
                            
                            //updates the gui
                            if(device.equals(Main.gui.getActiveDevice()) && Main.gui.getActiveIf().equals(tempwif.getName()))
                            {
                                Main.gui.refresh();
                            }                             
                        }    
                        
                        //if the value PER changed
                        if(!this.discWifTable.get(device+tempwif.getName())[3].equals(tempwif.getPacket_error_rate()))
                        {
                            //updates connected interface table
                            this.discWifTable.get(device+tempwif.getName())[3] = tempwif.getPacket_error_rate();
                            
                            //updates the database
                            this.updateData("WirelessData", "PER", tempwif.getPacket_error_rate(), device, tempwif.getName()); 
                            
                            //updates the gui
                            if(device.equals(Main.gui.getActiveDevice()) && Main.gui.getActiveIf().equals(tempwif.getName()))
                            {
                                Main.gui.refresh();
                            }                             
                        }                       
                    }
                    else if (this.conWifTable.containsKey(device + tempwif.getName()) && existed) //was connected but now is disconnected
                    {                              
                        //removes it from the connected list
                        this.conWifTable.remove(device + tempwif.getName());
                        
                        String[] temparray = new String[4];                        
                        //initiates the values in the table
                        temparray[0] = tempwif.getMac_address();
                        temparray[1] = tempwif.getMax_bitrate(); 
                        temparray[2] = tempwif.getCurrent_bitrate(); 
                        temparray[3] = tempwif.getPacket_error_rate(); 
                        
                        //adds it in the disconnected table
                        this.discWifTable.put(device + tempwif.getName(), temparray);                                             
                        
                        //changes the values in the database
                        this.updateData("WirelessData", "IpAddr", tempwif.getIp_address(), device, tempwif.getName());
                        this.updateData("WirelessData", "SubMask", tempwif.getSubnetmask(), device, tempwif.getName());
                        this.updateData("WirelessData", "NetAddr", tempwif.getNetwork_address(), device, tempwif.getName());
                        this.updateData("WirelessData", "BcastAddr", tempwif.getBroadcast_address(), device, tempwif.getName());
                        this.updateData("WirelessData", "DefGateway", tempwif.getDefault_gateway(), device, tempwif.getName());
                        this.updateData("WirelessData", "MaxBitRate", tempwif.getMax_bitrate(), device, tempwif.getName());
                        this.updateData("WirelessData", "CurBitRate", tempwif.getCurrent_bitrate(), device, tempwif.getName());
                        this.updateData("WirelessData", "CurUsedRange", tempwif.getCurrent_usedrange(), device, tempwif.getName());
                        this.updateData("WirelessData", "PER", tempwif.getPacket_error_rate(), device, tempwif.getName());
                        this.updateData("WirelessData", "ApMacAddr", tempwif.getAp_mac_address(), device, tempwif.getName());
                        this.updateData("WirelessData", "TPower", tempwif.getTransmit_power(), device, tempwif.getName());
                        this.updateData("WirelessData", "LinkQuality", tempwif.getLink_quality(), device, tempwif.getName());
                        this.updateData("WirelessData", "SignalLevel", tempwif.getSignal_level(), device, tempwif.getName());
                        this.updateData("WirelessData", "NoiseLevel", tempwif.getNoise_level(), device, tempwif.getName());
                        this.updateData("WirelessData", "DiscPacks", tempwif.getDiscarded_packets(), device, tempwif.getName());     
                        
                        //updates the gui
                        if(device.equals(Main.gui.getActiveDevice()) && Main.gui.getActiveIf().equals(tempwif.getName()))
                        {
                            Main.gui.refresh();
                        }                         
                    }
                    else //doesnt exist in any table
                    {
                        String[] temparray = new String[4];                        
                        //initiates the values in the table
                        temparray[0] = tempwif.getMac_address();
                        temparray[1] = tempwif.getMax_bitrate(); 
                        temparray[2] = tempwif.getCurrent_bitrate(); 
                        temparray[3] = tempwif.getPacket_error_rate(); 
                        
                        //adds it in the disconnected table
                        this.discWifTable.put(device + tempwif.getName(), temparray);
                        
                        newarrival = true;                        
                    }
                }
                else //is connected
                {
                    if (this.conWifTable.containsKey(device + tempwif.getName()) && existed) //already exists in the connected intefaces table
                    {
                        //if the value MacAddr changed
                        if(!this.conWifTable.get(device+tempwif.getName())[0].equals(tempwif.getMac_address()))
                        {
                            //updates connected interface table
                            this.conWifTable.get(device+tempwif.getName())[0] = tempwif.getMac_address();
                            
                            //updates the database
                            this.updateData("WirelessData", "MacAddr", tempwif.getMac_address(), device, tempwif.getName());   
                            
                            //updates the gui
                            if(device.equals(Main.gui.getActiveDevice()) && Main.gui.getActiveIf().equals(tempwif.getName()))
                            {
                                Main.gui.refresh();
                            }                             
                        }
                        
                        //if one of the values IpAddr or SubMask changed
                        if(!this.conWifTable.get(device+tempwif.getName())[1].equals(tempwif.getIp_address()) 
                           || !this.conWifTable.get(device+tempwif.getName())[2].equals(tempwif.getSubnetmask()))
                        {
                            //updates connected interface table
                            this.conWifTable.get(device+tempwif.getName())[1] = tempwif.getIp_address();
                            
                            //updates the database
                            this.updateData("WirelessData", "IpAddr", tempwif.getIp_address(), device, tempwif.getName());     
                            
                            //updates connected interface table
                            this.conWifTable.get(device+tempwif.getName())[2] = tempwif.getSubnetmask();
                            
                            //updates the database
                            this.updateData("WirelessData", "SubMask", tempwif.getSubnetmask(), device, tempwif.getName());
                            
                            //updates the network and broadcast addresses in the database
                            this.updateData("WirelessData", "NetAddr", tempwif.getNetwork_address(), device, tempwif.getName());
                            this.updateData("WirelessData", "BcastAddr", tempwif.getBroadcast_address(), device, tempwif.getName()); 
                            
                            //updates the gui
                            if(device.equals(Main.gui.getActiveDevice()) && Main.gui.getActiveIf().equals(tempwif.getName()))
                            {
                                Main.gui.refresh();
                            }                             
                        }
                        
                        //if the value DefGateway changed
                        if(!this.conWifTable.get(device+tempwif.getName())[3].equals(tempwif.getDefault_gateway()))
                        {
                            //updates connected interface table
                            this.conWifTable.get(device+tempwif.getName())[3] = tempwif.getDefault_gateway();
                            
                            //updates the database
                            this.updateData("WirelessData", "DefGateway", tempwif.getDefault_gateway(), device, tempwif.getName());   
                            
                            //updates the gui
                            if(device.equals(Main.gui.getActiveDevice()) && Main.gui.getActiveIf().equals(tempwif.getName()))
                            {
                                Main.gui.refresh();
                            }                             
                        }  
                        
                        //if one of the values MaxBitRate or CurBitRate changed
                        if(!this.conWifTable.get(device+tempwif.getName())[4].equals(tempwif.getMax_bitrate()) 
                           || !this.conWifTable.get(device+tempwif.getName())[5].equals(tempwif.getCurrent_bitrate()))
                        {
                            //updates connected interface table
                            this.conWifTable.get(device+tempwif.getName())[4] = tempwif.getMax_bitrate();
                            
                            //updates the database
                            this.updateData("WirelessData", "MaxBitRate", tempwif.getMax_bitrate(), device, tempwif.getName());    
                            
                            //updates connected interface table
                            this.conWifTable.get(device+tempwif.getName())[5] = tempwif.getCurrent_bitrate();
                            
                            //updates the database
                            this.updateData("WirelessData", "CurBitRate", tempwif.getCurrent_bitrate(), device, tempwif.getName());
                            
                            //updates the CurUsedRange in the database
                            this.updateData("WirelessData", "CurUsedRange", tempwif.getCurrent_usedrange(), device, tempwif.getName());  
                            
                            //updates the gui
                            if(device.equals(Main.gui.getActiveDevice()) && Main.gui.getActiveIf().equals(tempwif.getName()))
                            {
                                Main.gui.refresh();
                            }                             
                        }    
                        
                        //if the value PER changed
                        if(!this.conWifTable.get(device+tempwif.getName())[6].equals(tempwif.getPacket_error_rate()))
                        {
                            //updates connected interface table
                            this.conWifTable.get(device+tempwif.getName())[6] = tempwif.getPacket_error_rate();
                            
                            //updates the database
                            this.updateData("WirelessData", "PER", tempwif.getPacket_error_rate(), device, tempwif.getName());   
                            
                            //updates the gui
                            if(device.equals(Main.gui.getActiveDevice()) && Main.gui.getActiveIf().equals(tempwif.getName()))
                            {
                                Main.gui.refresh();
                            }                             
                        } 
                        
                        //if the value ApMacAddr changed
                        if(!this.conWifTable.get(device+tempwif.getName())[7].equals(tempwif.getAp_mac_address()))
                        {
                            //updates connected interface table
                            this.conWifTable.get(device+tempwif.getName())[7] = tempwif.getAp_mac_address();
                            
                            //updates the database
                            this.updateData("WirelessData", "ApMacAddr", tempwif.getAp_mac_address(), device, tempwif.getName());    
                            
                            //updates the gui
                            if(device.equals(Main.gui.getActiveDevice()) && Main.gui.getActiveIf().equals(tempwif.getName()))
                            {
                                Main.gui.refresh();
                            }                             
                        }   
                        
                        //if the value TPower changed
                        if(!this.conWifTable.get(device+tempwif.getName())[8].equals(tempwif.getTransmit_power()))
                        {
                            //updates connected interface table
                            this.conWifTable.get(device+tempwif.getName())[8] = tempwif.getTransmit_power();
                            
                            //updates the database
                            this.updateData("WirelessData", "TPower", tempwif.getTransmit_power(), device, tempwif.getName()); 
                            
                            //updates the gui
                            if(device.equals(Main.gui.getActiveDevice()) && Main.gui.getActiveIf().equals(tempwif.getName()))
                            {
                                Main.gui.refresh();
                            }                             
                        }    
                        
                        //if the value LinkQuality changed
                        if(!this.conWifTable.get(device+tempwif.getName())[9].equals(tempwif.getLink_quality()))
                        {
                            //updates connected interface table
                            this.conWifTable.get(device+tempwif.getName())[9] = tempwif.getLink_quality();
                            
                            //updates the database
                            this.updateData("WirelessData", "LinkQuality", tempwif.getLink_quality(), device, tempwif.getName());
                            
                            //updates the gui
                            if(device.equals(Main.gui.getActiveDevice()) && Main.gui.getActiveIf().equals(tempwif.getName()))
                            {
                                Main.gui.refresh();
                            }                             
                        } 
                        
                        //if the value SignalLevel changed
                        if(!this.conWifTable.get(device+tempwif.getName())[10].equals(tempwif.getSignal_level()))
                        {
                            //updates connected interface table
                            this.conWifTable.get(device+tempwif.getName())[10] = tempwif.getSignal_level();
                            
                            //updates the database
                            this.updateData("WirelessData", "SignalLevel", tempwif.getSignal_level(), device, tempwif.getName()); 
                            
                            //updates the gui
                            if(device.equals(Main.gui.getActiveDevice()) && Main.gui.getActiveIf().equals(tempwif.getName()))
                            {
                                Main.gui.refresh();
                            }                             
                        } 
                        
                        //if the value NoiseLevel changed
                        if(!this.conWifTable.get(device+tempwif.getName())[11].equals(tempwif.getNoise_level()))
                        {
                            //updates connected interface table
                            this.conWifTable.get(device+tempwif.getName())[11] = tempwif.getNoise_level();
                            
                            //updates the database
                            this.updateData("WirelessData", "NoiseLevel", tempwif.getNoise_level(), device, tempwif.getName());   
                            
                            //updates the gui
                            if(device.equals(Main.gui.getActiveDevice()) && Main.gui.getActiveIf().equals(tempwif.getName()))
                            {
                                Main.gui.refresh();
                            } 
                        } 
                        
                        //if the value DiscPacks changed
                        if(!this.conWifTable.get(device+tempwif.getName())[12].equals(tempwif.getDiscarded_packets()))
                        {
                            //updates connected interface table
                            this.conWifTable.get(device+tempwif.getName())[12] = tempwif.getDiscarded_packets();
                            
                            //updates the database
                            this.updateData("WirelessData", "DiscPacks", tempwif.getDiscarded_packets(), device, tempwif.getName());  
                            
                            //updates the gui
                            if(device.equals(Main.gui.getActiveDevice()) && Main.gui.getActiveIf().equals(tempwif.getName()))
                            {
                                Main.gui.refresh();
                            }                             
                        }                        
                    }
                    else if (this.discWifTable.containsKey(device + tempwif.getName()) && existed) //was disconnected but now is connected
                    {
                        //removes it from the disconnected list
                        this.discWifTable.remove(device + tempwif.getName());
                        
                        String[] temparray = new String[13];                        
                        //initiates the values in the table
                        temparray[0] = tempwif.getMac_address();
                        temparray[1] = tempwif.getIp_address(); 
                        temparray[2] = tempwif.getSubnetmask();
                        temparray[3] = tempwif.getDefault_gateway();
                        temparray[4] = tempwif.getMax_bitrate();
                        temparray[5] = tempwif.getCurrent_bitrate();
                        temparray[6] = tempwif.getPacket_error_rate();
                        temparray[7] = tempwif.getAp_mac_address();
                        temparray[8] = tempwif.getTransmit_power();
                        temparray[9] = tempwif.getLink_quality();
                        temparray[10] = tempwif.getSignal_level();
                        temparray[11] = tempwif.getNoise_level();
                        temparray[12] = tempwif.getDiscarded_packets();
                        
                        //adds it in the connected table
                        this.conWifTable.put(device + tempwif.getName(), temparray);                                             
                        
                        //changes the values in the database
                        this.updateData("WirelessData", "IpAddr", tempwif.getIp_address(), device, tempwif.getName());
                        this.updateData("WirelessData", "SubMask", tempwif.getSubnetmask(), device, tempwif.getName());
                        this.updateData("WirelessData", "NetAddr", tempwif.getNetwork_address(), device, tempwif.getName());
                        this.updateData("WirelessData", "BcastAddr", tempwif.getBroadcast_address(), device, tempwif.getName());
                        this.updateData("WirelessData", "DefGateway", tempwif.getDefault_gateway(), device, tempwif.getName());
                        this.updateData("WirelessData", "MaxBitRate", tempwif.getMax_bitrate(), device, tempwif.getName());
                        this.updateData("WirelessData", "CurBitRate", tempwif.getCurrent_bitrate(), device, tempwif.getName());
                        this.updateData("WirelessData", "CurUsedRange", tempwif.getCurrent_usedrange(), device, tempwif.getName());
                        this.updateData("WirelessData", "PER", tempwif.getPacket_error_rate(), device, tempwif.getName());
                        this.updateData("WirelessData", "ApMacAddr", tempwif.getAp_mac_address(), device, tempwif.getName());
                        this.updateData("WirelessData", "TPower", tempwif.getTransmit_power(), device, tempwif.getName());
                        this.updateData("WirelessData", "LinkQuality", tempwif.getLink_quality(), device, tempwif.getName());
                        this.updateData("WirelessData", "SignalLevel", tempwif.getSignal_level(), device, tempwif.getName());
                        this.updateData("WirelessData", "NoiseLevel", tempwif.getNoise_level(), device, tempwif.getName());
                        this.updateData("WirelessData", "DiscPacks", tempwif.getDiscarded_packets(), device, tempwif.getName());      
                        
                        //updates the gui
                        if(device.equals(Main.gui.getActiveDevice()) && Main.gui.getActiveIf().equals(tempwif.getName()))
                        {
                            Main.gui.refresh();
                        }                         
                    }
                    else //doesnt exist in any table
                    {
                        String[] temparray = new String[13];                        
                        //initiates the values in the table
                        temparray[0] = tempwif.getMac_address();
                        temparray[1] = tempwif.getIp_address(); 
                        temparray[2] = tempwif.getSubnetmask();
                        temparray[3] = tempwif.getDefault_gateway();
                        temparray[4] = tempwif.getMax_bitrate();
                        temparray[5] = tempwif.getCurrent_bitrate();
                        temparray[6] = tempwif.getPacket_error_rate();
                        temparray[7] = tempwif.getAp_mac_address();
                        temparray[8] = tempwif.getTransmit_power();
                        temparray[9] = tempwif.getLink_quality();
                        temparray[10] = tempwif.getSignal_level();
                        temparray[11] = tempwif.getNoise_level();
                        temparray[12] = tempwif.getDiscarded_packets();
                        
                        //adds it in the connected table
                        this.conWifTable.put(device + tempwif.getName(), temparray); 
                        
                        newarrival = true;                         
                    }                    
                }

                if (newarrival)
                {
                    pst = con.prepareStatement("INSERT INTO WirelessData(DeviceName, IfName, MacAddr, IpAddr, SubMask, "
                            + "NetAddr, BcastAddr, DefGateway, MaxBitRate, CurBitRate, CurUsedRange, PER, "
                            + "ApMacAddr, TPower, LinkQuality, SignalLevel, NoiseLevel, DiscPacks)"
                            + "VALUES(?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)");
                    pst.setString(1, device);
                    pst.setString(2, tempwif.getName());
                    pst.setString(3, tempwif.getMac_address());
                    pst.setString(4, tempwif.getIp_address());
                    pst.setString(5, tempwif.getSubnetmask());
                    pst.setString(6, tempwif.getNetwork_address());
                    pst.setString(7, tempwif.getBroadcast_address());
                    pst.setString(8, tempwif.getDefault_gateway());
                    pst.setString(9, tempwif.getMax_bitrate());
                    pst.setString(10, tempwif.getCurrent_bitrate());
                    pst.setString(11, tempwif.getCurrent_usedrange());
                    pst.setString(12, tempwif.getPacket_error_rate());
                    pst.setString(13, tempwif.getAp_mac_address());
                    pst.setString(14, tempwif.getTransmit_power());
                    pst.setString(15, tempwif.getLink_quality());
                    pst.setString(16, tempwif.getSignal_level());
                    pst.setString(17, tempwif.getNoise_level());
                    pst.setString(18, tempwif.getDiscarded_packets());                    
                    pst.executeUpdate();
                    pst.close();
                    
                    if(existed && Main.gui.getActiveDevice().equals(device))
                    {
                        Main.gui.addWireless(tempwif.getName());
                    }

                    newarrival = false;
                }
                
                ifexist.add(device + tempwif.getName());
            }
            
            for(Iterator<Map.Entry<String, String[]>> it = discWifTable.entrySet().iterator(); it.hasNext(); )
            {
                Map.Entry<String, String[]> entry = it.next();
                if (!ifexist.contains(entry.getKey()) && entry.getKey().startsWith(device))
                {                                       
                    this.deleteWireless(device, entry.getKey().replace(device, ""));
                    if(Main.gui.getActiveDevice().equals(device))
                    {
                        Main.gui.removeInterface(device, entry.getKey().replace(device, ""));
                    }
                    it.remove();                    
                }
            } 
            for(Iterator<Map.Entry<String, String[]>> it = conWifTable.entrySet().iterator(); it.hasNext(); )
            {
                Map.Entry<String, String[]> entry = it.next();
                if (!ifexist.contains(entry.getKey()) && entry.getKey().startsWith(device))
                {                                       
                    this.deleteWireless(device, entry.getKey().replace(device, ""));
                    if(Main.gui.getActiveDevice().equals(device))
                    {
                        Main.gui.removeInterface(device, entry.getKey().replace(device, ""));
                    }
                    it.remove();                    
                }
            } 
            ifexist.removeAll(ifexist);
            
            for(i = 0; i < MD.getAccessPointList().size(); i++)
            {
                tempap = MD.getAccessPointList().get(i);
                
                if (this.apTable.containsKey(tempap.getMac_address())) //access point already exists in access points table 
                {
                    //if the value Essid changed
                    if(!this.apTable.get(tempap.getMac_address())[0].equals(tempap.getEssid()))
                    {
                        //updates access points table
                        this.apTable.get(tempap.getMac_address())[0] = tempap.getEssid();

                        //updates the database
                        pst = con.prepareStatement("UPDATE AccessPointsData SET Essid = ? WHERE MacAddr = ?");
                        pst.setString(1, tempap.getEssid());
                        pst.setString(2, tempap.getMac_address());
                        pst.executeUpdate();
                        pst.close();  
                        
                        ////updates the gui
                        if(device.equals(Main.gui.getActiveDevice()) && Main.gui.getActiveIf().equals("Scanned APs"))
                        {
                            Main.gui.refresh();
                        }                                             
                    }     
                    
                    //if the value Channel changed
                    if(!this.apTable.get(tempap.getMac_address())[1].equals(tempap.getChannel()))
                    {
                        //updates access points table
                        this.apTable.get(tempap.getMac_address())[1] = tempap.getChannel();

                        //updates the database
                        pst = con.prepareStatement("UPDATE AccessPointsData SET Channel = ? WHERE MacAddr = ?");
                        pst.setString(1, tempap.getChannel());
                        pst.setString(2, tempap.getMac_address());
                        pst.executeUpdate();
                        pst.close();  
                        
                        ////updates the gui
                        if(device.equals(Main.gui.getActiveDevice()) && Main.gui.getActiveIf().equals("Scanned APs"))
                        {
                            Main.gui.refresh();
                        }                         
                    }
                    
                    //if the value Mode changed
                    if(!this.apTable.get(tempap.getMac_address())[2].equals(tempap.getMode()))
                    {
                        //updates access points table
                        this.apTable.get(tempap.getMac_address())[2] = tempap.getMode();

                        //updates the database
                        pst = con.prepareStatement("UPDATE AccessPointsData SET Mode = ? WHERE MacAddr = ?");
                        pst.setString(1, tempap.getMode());
                        pst.setString(2, tempap.getMac_address());
                        pst.executeUpdate();
                        pst.close();  
                        
                         ////updates the gui
                        if(device.equals(Main.gui.getActiveDevice()) && Main.gui.getActiveIf().equals("Scanned APs"))
                        {
                            Main.gui.refresh();
                        }                        
                    }                     
                }
                else //access point doesnt exist in the access points table
                {
                    String[] temparray = new String[3];
                    //initiates the values in the table
                    temparray[0] = tempap.getEssid();
                    temparray[1] = tempap.getChannel();
                    temparray[2] = tempap.getMode();
                    
                    //adds it into the access point table
                    this.apTable.put(tempap.getMac_address(), temparray);                    
                    
                    //adds it into the database
                    pst = con.prepareStatement("INSERT INTO AccessPointsData(MacAddr, Essid, Channel, Mode) "
                            + "VALUES(?,?,?,?)");
                    pst.setString(1, tempap.getMac_address());
                    pst.setString(2, tempap.getEssid());
                    pst.setString(3, tempap.getChannel());
                    pst.setString(4, tempap.getMode());                    
                    pst.executeUpdate();
                    pst.close();                    
                }
                
                if (this.devapTable.containsKey(device+tempap.getMac_address()) && existed) //already exists in devices' access points table 
                {
                    //if the value Signal Level has changed
                    if (!this.devapTable.get(device+tempap.getMac_address()).equals(tempap.getSignal_level()))
                    {
                        //updates devices' access points table
                        this.devapTable.put(device+tempap.getMac_address(), tempap.getSignal_level());

                        //updates the database
                        pst = con.prepareStatement("UPDATE DeviceAccessPoints SET SignalLevel = ? WHERE DeviceName = ? AND ApMacAddr = ?");
                        pst.setString(1, tempap.getSignal_level());
                        pst.setString(2, device);
                        pst.setString(3, tempap.getMac_address());
                        pst.executeUpdate();
                        pst.close(); 
                        
                        ////updates the gui
                        if(device.equals(Main.gui.getActiveDevice()) && Main.gui.getActiveIf().equals("Scanned APs"))
                        {
                            Main.gui.refresh();
                        }                         
                    }
                }
                else //doesnt exist in devices' access points table 
                {
                    //adds it into the devices' access points table
                    this.devapTable.put(device+tempap.getMac_address(), tempap.getSignal_level());
                    
                    //adds it into the database
                    pst = con.prepareStatement("INSERT INTO DeviceAccessPoints(DeviceName, ApMacAddr, SignalLevel) "
                            + "VALUES(?,?,?)");
                    pst.setString(1, device);
                    pst.setString(2, tempap.getMac_address());
                    pst.setString(3, tempap.getSignal_level());                   
                    pst.executeUpdate();
                    pst.close();
                    
                    ////updates the gui
                    if(device.equals(Main.gui.getActiveDevice()) && Main.gui.getActiveIf().equals("Scanned APs"))
                    {
                        Main.gui.refresh();
                    }                                      
                }                
                ifexist.add(device + tempap.getMac_address());
            }
            
            for(Iterator<Map.Entry<String, String>> it = devapTable.entrySet().iterator(); it.hasNext(); )
            {
                Map.Entry<String, String> entry = it.next();
                if (!ifexist.contains(entry.getKey()) && entry.getKey().startsWith(device))
                {                                       
                    this.deleteAccessPoint(device, entry.getKey().replace(device, ""));
                    if(Main.gui.getActiveDevice().equals(device) && Main.gui.getActiveIf().equals("Scanned APs"))
                    {
                        Main.gui.removeAccessPoint(device, entry.getKey().replace(device, ""));
                        Main.gui.refresh();
                    }
                    it.remove();                    
                }
            }

        } catch (SQLException ex)
        {
            Logger lgr = Logger.getLogger(DatabaseActions.class.getName());
            lgr.log(Level.SEVERE, ex.getMessage(), ex);

        } finally
        {
            try
            {
                if (pst != null)
                {
                    pst.close();
                }

            } catch (SQLException ex)
            {
                Logger lgr = Logger.getLogger(DatabaseActions.class.getName());
                lgr.log(Level.SEVERE, ex.getMessage(), ex);
            }
        }
    }
    
    private synchronized void updateData(String tableName, String dbElemName, String elem, String device, String name )
    {
        try
        {
            pst = con.prepareStatement("UPDATE "+tableName+" SET "+dbElemName+" = ? WHERE DeviceName = ? AND IfName = ?");
            pst.setString(1, elem);
            pst.setString(2, device);
            pst.setString(3, name);
            pst.executeUpdate();                      
        } catch (SQLException ex)
        {
            Logger.getLogger(DatabaseActions.class.getName()).log(Level.SEVERE, null, ex);
        } finally
        {
            try
            {
                if (pst != null)
                {
                    pst.close();
                }
            } catch (SQLException ex)
            {
                Logger lgr = Logger.getLogger(DatabaseActions.class.getName());
                lgr.log(Level.WARNING, ex.getMessage(), ex);
            }
        }
    }
    
    private synchronized void updateTerminalData( String dbElemName, String elem, String device)
    {
        try
        {
            pst = con.prepareStatement("UPDATE TerminalData SET "+dbElemName+" = ? WHERE IMEI = ?");
            pst.setString(1, elem);
            pst.setString(2, device);
            pst.executeUpdate();                      
        } catch (SQLException ex)
        {
            Logger.getLogger(DatabaseActions.class.getName()).log(Level.SEVERE, null, ex);
        } finally
        {
            try
            {
                if (pst != null)
                {
                    pst.close();
                }
            } catch (SQLException ex)
            {
                Logger lgr = Logger.getLogger(DatabaseActions.class.getName());
                lgr.log(Level.WARNING, ex.getMessage(), ex);
            }
        }
    }    
    
    
    /**Checks the database for the available interfaces for this device.
     * @param device The name of the device that we want to check its interfaces
     * @return A vector with the available interfaces for this device*/
    public synchronized Vector<String> getInterfaces(String device)
    {
        Vector<String> results = new Vector();
        try
        {
            pst = con.prepareStatement("SELECT IfName FROM InterfaceData WHERE DeviceName = ?");
            pst.setString(1, device);
            rs = pst.executeQuery();
                        
            while (rs.next())
            {
                results.add(rs.getString(1));
            }

        } catch (SQLException ex)
        {
            Logger lgr = Logger.getLogger(DatabaseActions.class.getName());
            lgr.log(Level.SEVERE, ex.getMessage(), ex);

        } finally
        {

            try
            {
                if (rs != null)
                {
                    rs.close();
                }
                if (pst != null)
                {
                    pst.close();
                }

            } catch (SQLException ex)
            {
                Logger lgr = Logger.getLogger(DatabaseActions.class.getName());
                lgr.log(Level.WARNING, ex.getMessage(), ex);
            }
        }
        
        return results;
    }     
    
    /**Gets from the middle memory the requested interface data for the device. 
     * @param device The name of the device that we want to get the interface data
     * @param ifname The name of the interface we want to get data from
     * @return A vector with the requested interface data from this device*/
    public synchronized Vector<String> getInterfaceData(String device, String ifname)
    {
        Vector<String> results = new Vector();
        
        if(ifname.equals("lo")) //the requested interface is local loopback
        {
            results.add(device);
            results.add(ifname);
            results.add("Not-Available");
            results.add("127.0.0.1");
            results.add("255.0.0.0");
            results.add("127.0.0.0");
            results.add("127.255.255.255");
            results.add("Not-Available");
            results.add("Not-Available");
            results.add(this.localTable.get(device));                       
            results.add("Not-Available");   
            
            try
            {
                pst = con.prepareStatement("SELECT PER FROM InterfaceData WHERE DeviceName = ? AND IfName = ?");
                pst.setString(1, device);
                pst.setString(2, ifname);
                rs = pst.executeQuery();
                
                rs.next();
                results.add(rs.getString(1));
            } catch (SQLException ex)
            {
                Logger lgr = Logger.getLogger(DatabaseActions.class.getName());
                lgr.log(Level.SEVERE, ex.getMessage(), ex);

            } finally
            {

                try
                {
                    if (rs != null)
                    {
                        rs.close();
                    }
                    if (pst != null)
                    {
                        pst.close();
                    }

                } catch (SQLException ex)
                {
                    Logger lgr = Logger.getLogger(DatabaseActions.class.getName());
                    lgr.log(Level.WARNING, ex.getMessage(), ex);
                }
            }            
        }
        //the requested interface is a disconnected interface
        else if(this.discIfTable.containsKey(device+ifname))
        {
            results.add(device);
            results.add(ifname);
            results.add(this.discIfTable.get(device+ifname));
            results.add("Not-Available");
            results.add("Not-Available");
            results.add("Not-Available");
            results.add("Not-Available");
            results.add("Not-Available");
            results.add("0 kb/s");           
            results.add("0.0 %");
            results.add("0");
            
            try
            {
                pst = con.prepareStatement("SELECT PER,MaxBitRate FROM InterfaceData WHERE DeviceName = ? AND IfName = ?");
                pst.setString(1, device);
                pst.setString(2, ifname);
                rs = pst.executeQuery();
                
                rs.next();
                results.add(rs.getString(1));
                results.add(8, rs.getString(2));
            } catch (SQLException ex)
            {
                Logger lgr = Logger.getLogger(DatabaseActions.class.getName());
                lgr.log(Level.SEVERE, ex.getMessage(), ex);

            } finally
            {

                try
                {
                    if (rs != null)
                    {
                        rs.close();
                    }
                    if (pst != null)
                    {
                        pst.close();
                    }

                } catch (SQLException ex)
                {
                    Logger lgr = Logger.getLogger(DatabaseActions.class.getName());
                    lgr.log(Level.WARNING, ex.getMessage(), ex);
                }
            }
        }//the requested interface is a connected interface
        else if(this.conIfTable.containsKey(device+ifname))
        {
            results.add(device);
            results.add(ifname);
            results.add(this.conIfTable.get(device+ifname)[0]);
            results.add(this.conIfTable.get(device+ifname)[1]);
            results.add(this.conIfTable.get(device+ifname)[2]);
            
            try
            {
                pst = con.prepareStatement("SELECT NetAddr,BcastAddr FROM InterfaceData WHERE DeviceName = ? AND IfName = ?");
                pst.setString(1, device);
                pst.setString(2, ifname);
                rs = pst.executeQuery();
                
                rs.next();
                results.add(rs.getString(1));
                results.add(rs.getString(2));
            } catch (SQLException ex)
            {
                Logger lgr = Logger.getLogger(DatabaseActions.class.getName());
                lgr.log(Level.SEVERE, ex.getMessage(), ex);

            } finally
            {

                try
                {
                    if (rs != null)
                    {
                        rs.close();
                    }
                    if (pst != null)
                    {
                        pst.close();
                    }

                } catch (SQLException ex)
                {
                    Logger lgr = Logger.getLogger(DatabaseActions.class.getName());
                    lgr.log(Level.WARNING, ex.getMessage(), ex);
                }
            }             
            
            results.add(this.conIfTable.get(device+ifname)[3]);
            results.add(this.conIfTable.get(device+ifname)[4]);
            results.add(this.conIfTable.get(device+ifname)[5]);
            double r = (Double.parseDouble(this.conIfTable.get(device+ifname)[5].replaceFirst(" kb/s", ""))/
                                        Double.parseDouble(this.conIfTable.get(device+ifname)[4].replaceFirst(" kb/s", "")))
                                        *100.0;              
            results.add(Double.toString(r).concat(" %"));
            results.add(this.conIfTable.get(device+ifname)[6]);                   
        }
        return results;       
    }    
  
    /**Deletes from the database the requested interface for the device.
     * @param device The name of the device that the interface belongs
     * @param ifname The name of the interface we want to delete*/             
    public synchronized void deleteInterface(String device, String ifname)
    {
        try
        {
            pst = con.prepareStatement("DELETE FROM InterfaceData WHERE DeviceName = ? AND IfName = ?");
            pst.setString(1, device);
            pst.setString(2, ifname);
            pst.executeUpdate();            
        } catch (SQLException ex)
        {
            Logger lgr = Logger.getLogger(DatabaseActions.class.getName());
            lgr.log(Level.SEVERE, ex.getMessage(), ex);

        } finally
        {
            try
            {
                if (pst != null)
                {
                    pst.close();
                }

            } catch (SQLException ex)
            {
                Logger lgr = Logger.getLogger(DatabaseActions.class.getName());
                lgr.log(Level.WARNING, ex.getMessage(), ex);
            }
        }        
    }  
    
    /**Checks the database for the available wireless interfaces for this device.
     * @param device The name of the device that we want to check its wireless interfaces
     * @return A vector with the available wireless interfaces for this device*/
    public synchronized Vector<String> getWireless(String device)
    {
        Vector<String> results = new Vector();
        try
        {
            pst = con.prepareStatement("SELECT IfName FROM WirelessData WHERE DeviceName = ?");
            pst.setString(1, device);
            rs = pst.executeQuery();
                        
            while (rs.next())
            {
                results.add(rs.getString(1));
            }

        } catch (SQLException ex)
        {
            Logger lgr = Logger.getLogger(DatabaseActions.class.getName());
            lgr.log(Level.SEVERE, ex.getMessage(), ex);

        } finally
        {

            try
            {
                if (rs != null)
                {
                    rs.close();
                }
                if (pst != null)
                {
                    pst.close();
                }

            } catch (SQLException ex)
            {
                Logger lgr = Logger.getLogger(DatabaseActions.class.getName());
                lgr.log(Level.WARNING, ex.getMessage(), ex);
            }
        }
        
        return results;
    }    
    
    /**Gets from the middle memory the requested wireless interface data for the device. 
     * @param device The name of the device that we want to get the wireless interface data
     * @param wifname The name of the wireless interface we want to get data from
     * @return A vector with the requested wireless data from this device*/
    public synchronized Vector<String> getWirelessData(String device, String wifname)
    {
        Vector<String> results = new Vector();
        
        if(this.discWifTable.containsKey(device+wifname)) //the requested wireless interface is disconnected
        {
            results.add(device);
            results.add(wifname);
            results.add(this.discWifTable.get(device+wifname)[0]);
            results.add("Not-Available");
            results.add("Not-Available");
            results.add("Not-Available");
            results.add("Not-Available");
            results.add("Not-Available");
            results.add(this.discWifTable.get(device+wifname)[1]);
            results.add(this.discWifTable.get(device+wifname)[2]);
            if(this.discWifTable.get(device+wifname)[1].equals("Not-available") || this.discWifTable.get(device+wifname)[2].equals("Not-available"))
            {
                results.add("Not-available");
            }
            else
            {
                double r = (Double.parseDouble(this.discWifTable.get(device+wifname)[2].replaceFirst(" kb/s", ""))/
                                            Double.parseDouble(this.discWifTable.get(device+wifname)[1].replaceFirst(" kb/s", "")))
                                            *100.0;            
                results.add(Double.toString(r).concat(" %"));               
            }
            results.add(this.discWifTable.get(device+wifname)[3]);
            results.add("Not-Associated");
            results.add("Not-Available");
            results.add("Not-Available");
            results.add("Not-Available");
            results.add("Not-Available");
            results.add("Not-Available");
            results.add("Not-Available");
            results.add("Not-Available");
            try
            {
                pst = con.prepareStatement("SELECT DiscPacks FROM WirelessData WHERE DeviceName = ? AND IfName = ?");
                pst.setString(1, device);
                pst.setString(2, wifname);
                rs = pst.executeQuery();
                
                rs.next();
                results.add(rs.getString(1));
            } catch (SQLException ex)
            {
                Logger lgr = Logger.getLogger(DatabaseActions.class.getName());
                lgr.log(Level.SEVERE, ex.getMessage(), ex);

            } finally
            {

                try
                {
                    if (rs != null)
                    {
                        rs.close();
                    }
                    if (pst != null)
                    {
                        pst.close();
                    }

                } catch (SQLException ex)
                {
                    Logger lgr = Logger.getLogger(DatabaseActions.class.getName());
                    lgr.log(Level.WARNING, ex.getMessage(), ex);
                }
            }
        }
        else if(this.conWifTable.containsKey(device+wifname)) //the requested wireless interface is connected
        {
            results.add(device);
            results.add(wifname);
            results.add(this.conWifTable.get(device+wifname)[0]);
            results.add(this.conWifTable.get(device+wifname)[1]);
            results.add(this.conWifTable.get(device+wifname)[2]);
            
            try
            {
                pst = con.prepareStatement("SELECT NetAddr,BcastAddr FROM WirelessData WHERE DeviceName = ? AND IfName = ?");
                pst.setString(1, device);
                pst.setString(2, wifname);
                rs = pst.executeQuery();
                
                rs.next();
                results.add(rs.getString(1));
                results.add(rs.getString(2));
            } catch (SQLException ex)
            {
                Logger lgr = Logger.getLogger(DatabaseActions.class.getName());
                lgr.log(Level.SEVERE, ex.getMessage(), ex);

            } finally
            {

                try
                {
                    if (rs != null)
                    {
                        rs.close();
                    }
                    if (pst != null)
                    {
                        pst.close();
                    }

                } catch (SQLException ex)
                {
                    Logger lgr = Logger.getLogger(DatabaseActions.class.getName());
                    lgr.log(Level.WARNING, ex.getMessage(), ex);
                }
            }             
            
            results.add(this.conWifTable.get(device+wifname)[3]);
            results.add(this.conWifTable.get(device+wifname)[4]);
            results.add(this.conWifTable.get(device+wifname)[5]);
            double r = (Double.parseDouble(this.conWifTable.get(device+wifname)[5].replaceFirst(" kb/s", ""))/
                                        Double.parseDouble(this.conWifTable.get(device+wifname)[4].replaceFirst(" kb/s", "")))
                                        *100.0;              
            results.add(Double.toString(r).concat(" %"));
            results.add(this.conWifTable.get(device+wifname)[6]);
            results.add(this.conWifTable.get(device+wifname)[7]);
            results.add(this.apTable.get(this.conWifTable.get(device+wifname)[7])[0]);
            results.add(this.apTable.get(this.conWifTable.get(device+wifname)[7])[1]);
            results.add(this.apTable.get(this.conWifTable.get(device+wifname)[7])[2]);
            results.add(this.conWifTable.get(device+wifname)[8]);
            results.add(this.conWifTable.get(device+wifname)[9]);
            results.add(this.conWifTable.get(device+wifname)[10]);
            results.add(this.conWifTable.get(device+wifname)[11]);
            results.add(this.conWifTable.get(device+wifname)[12]);                   
        }
        return results;
    }    
    
    /**Deletes from the database the requested wireless interface for the device.
     * @param device The name of the device that the wireless interface belongs
     * @param wifname The name of the wireless interface we want to delete*/
    public synchronized void deleteWireless(String device, String wifname)
    {
        try
        {
            pst = con.prepareStatement("DELETE FROM WirelessData WHERE DeviceName = ? AND IfName = ?");
            pst.setString(1, device);
            pst.setString(2, wifname);
            pst.executeUpdate();            
        } catch (SQLException ex)
        {
            Logger lgr = Logger.getLogger(DatabaseActions.class.getName());
            lgr.log(Level.SEVERE, ex.getMessage(), ex);

        } finally
        {
            try
            {
                if (pst != null)
                {
                    pst.close();
                }

            } catch (SQLException ex)
            {
                Logger lgr = Logger.getLogger(DatabaseActions.class.getName());
                lgr.log(Level.WARNING, ex.getMessage(), ex);
            }
        }        
    } 
    
    /**Gets from the middle memory the available access points and their data for the device.
     * @param device The name of the device we want to get the discovered access points 
     * @return A list of String arrays where each array contains data from an access point*/
    public synchronized List<String[]> getAccessPointsData(String device)
    {
        String[] temparray;
        List<String[]> apList = new ArrayList();
        
        for (Map.Entry<String, String> entry : this.devapTable.entrySet())
        {
            String temp = entry.getKey().replace(device, "");
            if(entry.getKey().replace(temp, "").equals(device))
            {
                temparray = new String[5];
                temparray[0] = temp;
                temparray[4] = entry.getValue();
                apList.add(temparray);
            }
        }
        
        for (int i = 0; i < apList.size(); i++)
        {            
            apList.get(i)[1] = apTable.get(apList.get(i)[0])[0];
            apList.get(i)[2] = apTable.get(apList.get(i)[0])[1];
            apList.get(i)[3] = apTable.get(apList.get(i)[0])[2];
        }        
        
        return apList;
    }    
    
    /**Deletes the access point for the device from the database 
     * @param device The name of the device we want to delete the access point from
     * @param mac The mac address of the access point we want to delete*/
    public synchronized void deleteAccessPoint(String device, String mac)
    {
        try
        {
            pst = con.prepareStatement("DELETE FROM DeviceAccessPoints WHERE DeviceName = ? AND ApMacAddr = ?");
            pst.setString(1, device);
            pst.setString(2, mac);
            pst.executeUpdate();            
        } catch (SQLException ex)
        {
            Logger lgr = Logger.getLogger(DatabaseActions.class.getName());
            lgr.log(Level.SEVERE, ex.getMessage(), ex);

        } finally
        {
            try
            {
                if (pst != null)
                {
                    pst.close();
                }

            } catch (SQLException ex)
            {
                Logger lgr = Logger.getLogger(DatabaseActions.class.getName());
                lgr.log(Level.WARNING, ex.getMessage(), ex);
            }
        }        
    }                 
    
    /**Deletes all the collected data of the device from the database and the middle memory.
     * @param device The name of the device we want to delete*/
    private synchronized void deleteDevice(String device)
    {
        try
        {
            pst = con.prepareStatement("DELETE FROM InterfaceData WHERE DeviceName = ?");
            pst.setString(1, device);
            pst.executeUpdate();
            pst = con.prepareStatement("DELETE FROM WirelessData WHERE DeviceName = ?");
            pst.setString(1, device);
            pst.executeUpdate();
            pst = con.prepareStatement("DELETE FROM DeviceAccessPoints WHERE DeviceName = ?");
            pst.setString(1, device);            
            pst.executeUpdate();
            
            this.localTable.remove(device);
            
            for(Iterator<Map.Entry<String, String>> it = discIfTable.entrySet().iterator(); it.hasNext(); )
            {
                Map.Entry<String, String> entry = it.next();
                if (entry.getKey().startsWith(device))
                {                                       
                    it.remove();                    
                }
            }  
            for(Iterator<Map.Entry<String, String[]>> it = conIfTable.entrySet().iterator(); it.hasNext(); )
            {
                Map.Entry<String, String[]> entry = it.next();
                if (entry.getKey().startsWith(device))
                {                                       
                    it.remove();                    
                }
            } 
            
            for(Iterator<Map.Entry<String, String[]>> it = discWifTable.entrySet().iterator(); it.hasNext(); )
            {
                Map.Entry<String, String[]> entry = it.next();
                if (entry.getKey().startsWith(device))
                {                                       
                    it.remove();                    
                }
            } 
            for(Iterator<Map.Entry<String, String[]>> it = conWifTable.entrySet().iterator(); it.hasNext(); )
            {
                Map.Entry<String, String[]> entry = it.next();
                if (entry.getKey().startsWith(device))
                {                                       
                    it.remove();                    
                }
            }
            
            for(Iterator<Map.Entry<String, String>> it = devapTable.entrySet().iterator(); it.hasNext(); )
            {
                Map.Entry<String, String> entry = it.next();
                if (entry.getKey().startsWith(device))
                {                                       
                    it.remove();                    
                }
            }            
            
        } catch (SQLException ex)
        {
            Logger lgr = Logger.getLogger(DatabaseActions.class.getName());
            lgr.log(Level.SEVERE, ex.getMessage(), ex);

        } finally
        {
            try
            {
                if (pst != null)
                {
                    pst.close();
                }

            } catch (SQLException ex)
            {
                Logger lgr = Logger.getLogger(DatabaseActions.class.getName());
                lgr.log(Level.WARNING, ex.getMessage(), ex);
            }
        }        
    }    
    

    public Vector<String> getTerinalData(String device)
    {
        Vector<String> results = new Vector();
        try
        {
            pst = con.prepareStatement("SELECT Model, Manufacturer, Version FROM TerminalData WHERE IMEI = ?");
            pst.setString(1, device);
            rs = pst.executeQuery();
                        
            rs.next();
           
            results.add(rs.getString(1)); 
            results.add(rs.getString(2));
            results.add(rs.getString(3));
            results.add(this.termDataTable.get(device)[0]);
            results.add(this.termDataTable.get(device)[1]);
            results.add(this.termDataTable.get(device)[2]);
            results.add(this.termDataTable.get(device)[3]);

        } catch (SQLException ex)
        {
            Logger lgr = Logger.getLogger(DatabaseActions.class.getName());
            lgr.log(Level.SEVERE, ex.getMessage(), ex);

        } finally
        {

            try
            {
                if (rs != null)
                {
                    rs.close();
                }
                if (pst != null)
                {
                    pst.close();
                }

            } catch (SQLException ex)
            {
                Logger lgr = Logger.getLogger(DatabaseActions.class.getName());
                lgr.log(Level.WARNING, ex.getMessage(), ex);
            }
        }
        
        return results;
    }    
    
    /**Deletes all the collected data of the terminal from the database and the middle memory.
     * @param device The name of the terminal we want to delete*/
    private synchronized void deleteTerminal(String device)
    {
        try
        {
            pst = con.prepareStatement("DELETE FROM TerminalData WHERE IMEI = ?");
            pst.setString(1, device);
            pst.executeUpdate();
            
            for(Iterator<Map.Entry<String, String[]>> it = termDataTable.entrySet().iterator(); it.hasNext(); )
            {
                Map.Entry<String, String[]> entry = it.next();
                if (entry.getKey().equals(device))
                {                                       
                    it.remove();                    
                }
            }                        
        } catch (SQLException ex)
        {
            Logger lgr = Logger.getLogger(DatabaseActions.class.getName());
            lgr.log(Level.SEVERE, ex.getMessage(), ex);

        } finally
        {
            try
            {
                if (pst != null)
                {
                    pst.close();
                }

            } catch (SQLException ex)
            {
                Logger lgr = Logger.getLogger(DatabaseActions.class.getName());
                lgr.log(Level.WARNING, ex.getMessage(), ex);
            }
        }        
    }     

    /**
     *
     */
    private void deleteTables()
    {
        try
        {                       
            pst = con.prepareStatement("DROP TABLE IF EXISTS InterfaceData");
            pst.executeUpdate();
            
            pst = con.prepareStatement("DROP TABLE IF EXISTS WirelessData");
            pst.executeUpdate();
            
            pst = con.prepareStatement("DROP TABLE IF EXISTS DeviceAccessPoints");
            pst.executeUpdate();            
           
            pst = con.prepareStatement("DROP TABLE IF EXISTS AccessPointsData");
            pst.executeUpdate();
            
            pst = con.prepareStatement("DROP TABLE IF EXISTS TerminalData");
            pst.executeUpdate();            


        } catch (SQLException ex)
        {
            Logger.getLogger(DatabaseActions.class.getName()).log(Level.SEVERE, null, ex);
        } finally
        {
            try
            {
                if (pst != null)
                {
                    pst.close();
                }          
                
            } catch (SQLException ex)
            {
                Logger.getLogger(DatabaseActions.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
    }

    /**Finds the created tables in the current database and prints them. */
    private void showTables()
    {
        try
        {
            pst = con.prepareStatement("SHOW TABLES");
            rs = pst.executeQuery();

            while (rs.next())
            {

                System.out.println(rs.getString(1));
            }

        } catch (SQLException ex)
        {
            Logger lgr = Logger.getLogger(DatabaseActions.class.getName());
            lgr.log(Level.SEVERE, ex.getMessage(), ex);

        } finally
        {

            try
            {
                if (rs != null)
                {
                    rs.close();
                }
                if (pst != null)
                {
                    pst.close();
                }

            } catch (SQLException ex)
            {
                Logger lgr = Logger.getLogger(DatabaseActions.class.getName());
                lgr.log(Level.WARNING, ex.getMessage(), ex);
            }
        }
    }
    
    /**Closes the connection between the program and the database. */
    private synchronized void closeConection()
    {
        if (con != null)
        {
            try
            {
                this.con.close();
            } catch (SQLException ex)
            {
                Logger.getLogger(DatabaseActions.class.getName()).log(Level.SEVERE, null, ex);
            }
        }        
    }
    
    @Override
    public void run()
    {
        //gets from the properties file the sleeptime for the thread and the time limit to delete a device
        long s = Long.parseLong(Main.props.getProperty("sleeptime")), tl = Long.parseLong(Main.props.getProperty("timelimit"));
        
        while (true)
        {
            for(Iterator<Map.Entry<String, Long>> it = devtimesTable.entrySet().iterator(); it.hasNext(); )
            {
                Map.Entry<String, Long> entry = it.next();
                if (System.currentTimeMillis()-entry.getValue() > tl)
                {
                    this.deleteDevice(entry.getKey());
                    Main.gui.removeDevice(entry.getKey());
                    it.remove();                    
                }
            }
            
            for(Iterator<Map.Entry<String, Long>> it = termtimesTable.entrySet().iterator(); it.hasNext(); )
            {
                Map.Entry<String, Long> entry = it.next();
                if (System.currentTimeMillis()-entry.getValue() > tl)
                {
                    this.deleteTerminal(entry.getKey());
                    Main.gui.removeDevice(entry.getKey());
                    it.remove();                    
                }
            }            

            if (!Main.running) //if it gets false - termination signal
            {
                this.closeConection(); //closes the connection with the database
                break; //stops running
            }            
            
            try
            {
                sleep(s);
            } catch (InterruptedException ex)
            {
                Logger.getLogger(DatabaseActions.class.getName()).log(Level.SEVERE, null, ex);
            }
             
            if (!Main.running) //if it gets false - termination signal
            {
                this.closeConection(); //closes the connection with the database
                break; //stops running
            }
        }
    }
}
