
package guipack;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Vector;
import javax.swing.*;
import main.Main;


/**The UserInterface class that creates the graphical user interface,
 * contains all the methods to add and remove components, and has the 
 * necessary action listeners: one on this class, and three more in the 
 * inner classes that are contained. */
public class UserInterface extends JFrame implements ActionListener
{
    private volatile JPanel panel, north, nn, nc, west, wn, wc, center, cn, cc;
    /**Variables that show which device and which interface of this device are active at the gui. */
    private volatile String activeDevice = "", activeIf = "";
    /**Map that store the buttons of the representing devices. */
    private volatile Map<String, JButton> deviceTable = new HashMap();
    /**Map that store the buttons of the representing interfaces. */
    private volatile Map<String, JButton> ifTable = new HashMap();
    /**Map that store the buttons of the representing access points. */
    private volatile Map<String, JPanel> apTable = new HashMap();
    
    /**Constructor of the user interface that creates and sets up the frame. */
    public UserInterface()
    {
        super("WebService");
        this.setSize(900, 600);
        this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        this.setLayout(null);
        
        this.createPanels();       
        
        this.createPCLabel();        
       
        this.createInterfacesLabel();                      
        
        this.createDataLabel();
        
        this.setEnabled(true);
        this.setResizable(false);
        this.setVisible(true);
    }

    /**Getter for the active device.
     * @return The name of the active device */
    public String getActiveDevice()
    {
        return activeDevice;
    }

    /**Getter for the active interface.
     * @return The name of the active interface */
    public String getActiveIf()
    {
        return activeIf;
    }
    
    private void createPanels()
    {
        panel = new JPanel();
        panel.setBackground(Color.GRAY);
        BorderLayout pbl = new BorderLayout();
        pbl.setHgap(10);
        pbl.setVgap(10); 
        panel.setLayout(pbl);
        
        north = new JPanel();
        north.setBackground(Color.LIGHT_GRAY); 
        BorderLayout nbl = new BorderLayout();
        north.setLayout(nbl);
        
        nn = new JPanel();
        nn.setBackground(Color.WHITE);
        nn.setLayout(new FlowLayout(FlowLayout.CENTER));       
        
        nc = new JPanel();
        nc.setBackground(Color.LIGHT_GRAY);                        
        nc.setLayout(new FlowLayout(FlowLayout.CENTER));                
        
        north.add(BorderLayout.NORTH, nn);
        north.add(BorderLayout.CENTER, nc);
        north.add(BorderLayout.WEST, Box.createRigidArea(new Dimension(0, 53)));
        north.add(new JScrollPane(nc));       
        
        west = new JPanel();
        west.setBackground(Color.LIGHT_GRAY);         
        BorderLayout wbl = new BorderLayout();
        west.setLayout(wbl);
        
        wn = new JPanel();
        wn.setBackground(Color.WHITE);
        wn.setLayout(new FlowLayout(FlowLayout.CENTER));
        
        wc = new JPanel();
        wc.setBackground(Color.LIGHT_GRAY);        
        wc.setLayout(new BoxLayout(wc, BoxLayout.Y_AXIS));      
        
        west.add(BorderLayout.NORTH, wn);
        west.add(BorderLayout.CENTER, wc);        
        west.add(new JScrollPane(wc));
        //west.setVisible(false);
                
        center = new JPanel();
        center.setBackground(Color.LIGHT_GRAY); 
        BorderLayout cbl = new BorderLayout();
        center.setLayout(cbl);
        
        cn = new JPanel();
        cn.setBackground(Color.WHITE);
        cn.setLayout(new FlowLayout(FlowLayout.CENTER));       
        
        cc = new JPanel();
        cc.setBackground(Color.LIGHT_GRAY);                                        
        
        center.add(BorderLayout.NORTH, cn);
        center.add(BorderLayout.CENTER, cc);
        center.add(BorderLayout.WEST, Box.createRigidArea(new Dimension(25, 5)));
        center.add(BorderLayout.EAST, Box.createRigidArea(new Dimension(25, 5)));
        center.add(new JScrollPane(cc));         

        panel.add(BorderLayout.NORTH, north);
        panel.add(BorderLayout.WEST, west);
        panel.add(BorderLayout.CENTER, center);
                
        this.setContentPane(panel);
    }
    

    private void createPCLabel()
    { 
        Icon icon = new ImageIcon(getClass().getResource("pcIcon.png"));        
        JLabel jl = new JLabel("List of Devices:", icon, JLabel.CENTER);      
        nn.add(jl);
    } 
    
    private void createInterfacesLabel()
    {  
        Icon icon = new ImageIcon(getClass().getResource("interfacesIcon.png"));         
        JLabel jl = new JLabel("List of Interfaces:", icon, JLabel.CENTER);               
        wn.add(jl);
    } 
    
    private void createDataLabel()
    {  
        Icon icon = new ImageIcon(getClass().getResource("dataIcon.png"));         
        JLabel jl = new JLabel("Real-Time Data:", icon, JLabel.CENTER);               
        cn.add(jl);
    }
    
    /**Adds a new device to the user interface.
     * @param device The name of the device to be added to the gui*/
    public synchronized void addDevice(String device)
    {        
        JButton b = new JButton(device);   
        b.addActionListener(this);        
        nc.add(b);
        deviceTable.put(device, b);
        nc.revalidate();
        nc.repaint();
    }
    
    /**Adds a new terminal to the user interface.
     * @param device The name of the terminal to be added to the gui*/
    public synchronized void addTerminal(String device)
    {        
        JButton b = new JButton(device);   
        b.addActionListener(new TerminalActionListener());        
        nc.add(b);
        deviceTable.put(device, b);
        nc.revalidate();
        nc.repaint();
    }    
    
    /**Removes a device from the user interface.
     * @param device The name of the device to be removed from the gui*/
    public synchronized void removeDevice(String device)
    {
        JButton b = deviceTable.remove(device);
        if (device.equals(activeDevice))
        {
            wc.removeAll();            
            wc.revalidate();
            wc.repaint();
            
            cc.removeAll();
            cc.revalidate();
            cc.repaint();            
        }
        nc.remove(b);
        nc.revalidate();
        nc.repaint();
    }
    
    /**Adds a new interface to the user interface.
     * @param name The name of the interface to be added to the gui*/
    public synchronized void addInterface(String name)
    {
        wc.add(Box.createRigidArea(new Dimension(5, 5)));
        
        JButton b = new JButton(name); 
        b.addActionListener(new IfActionListener());
        b.setAlignmentX(Component.CENTER_ALIGNMENT);                 
        wc.add(b); 
        ifTable.put(name, b);
        wc.revalidate();        
        wc.repaint();        
    } 
    
    /**Adds a new wireless interface to the user interface.
     * @param name The name of the wireless interface to be added to the gui*/
    public synchronized void addWireless(String name)
    {
        wc.add(Box.createRigidArea(new Dimension(5, 5)));
        
        JButton b = new JButton(name);  
        b.addActionListener(new WifActionListener());
        b.setAlignmentX(Component.CENTER_ALIGNMENT);                 
        wc.add(b); 
        ifTable.put(name, b);
        wc.revalidate();        
        wc.repaint();        
    }                
    
    /**Removes an interface from the user interface.
     * @param device The name of the device that the interface belongs
     * @param ifname The name of the interface to be deleted*/
    public synchronized void removeInterface(String device, String ifname)
    {
        JButton b = ifTable.remove(ifname);
        if (ifname.equals(activeIf) && device.equals(activeDevice))
        {            
            cc.removeAll();
            cc.revalidate();
            cc.repaint();            
        }
        wc.remove(b);
        wc.revalidate();
        wc.repaint();
    }       
    
    /**Adds the Scanned APs to the user interface.
     * @param name The name of the scanner to be added to the gui*/    
    private synchronized void addScanner(String name)
    {
        wc.add(Box.createRigidArea(new Dimension(5, 5)));
        
        JButton b = new JButton(name); 
        b.addActionListener(new ApActionListener());
        b.setAlignmentX(Component.CENTER_ALIGNMENT);                 
        wc.add(b); 
        ifTable.put(name, b);
        wc.revalidate();        
        wc.repaint();        
    }    
    
    /**Adds a new access point to the results of the scanned access points shown on the gui.
     * @param mac The mac address of the access point 
     * @param values Array with the data to be added
     */
    public synchronized void addAccessPoint(String mac, String[] values)
    { 
        cc.add(Box.createRigidArea(new Dimension(1, 1)));
        
        JLabel temp;
        JPanel jp = new JPanel();
        jp.setBackground(Color.LIGHT_GRAY);
        GridLayout gl = new GridLayout(1,5,5,0);
        jp.setLayout(gl);
        for (int i = 0; i < 5; i++)
        {
            temp =  new JLabel(values[i]);
            temp.setOpaque(true);
            temp.setBackground(Color.WHITE);
            jp.add(temp);
        }
        cc.add(jp);
        if (mac != null)
        {
            apTable.put(mac, jp);            
        }
        else
        {
            cc.add(Box.createRigidArea(new Dimension(5, 5)));
        }
    }          

    /**Removes an access point from the results of the scanned access points shown on the gui.
     * @param device The name of the device
     * @param mac The mac address of the access point 
     */
    public synchronized void removeAccessPoint(String device, String mac)
    {          
        if (activeIf.equals("Scanned APs") && device.equals(activeDevice))
        {   
            JPanel jp = apTable.remove(mac);            
            cc.remove(jp);
            cc.revalidate();
            cc.repaint();            
        }
    }              
    
    /**Adds the interface data to the user interface.
     * @param name The name of the data to be added*/
    public synchronized void addInterfaceData(String name)
    {        
        JLabel jl = new JLabel(name);
        jl.setOpaque(true);
        jl.setBackground(Color.WHITE);
        jl.setAlignmentX(Component.CENTER_ALIGNMENT);
        jl.setAlignmentY(Component.CENTER_ALIGNMENT);
        cc.add(jl);        
    } 
    
    /**Refreshes the data that are currently shown. */
    public synchronized void refresh()
    {
        this.ifTable.get(activeIf).doClick(0);
    }
    
    /**Refreshes the data that are currently shown. */
    public synchronized void refreshTerminal()
    {
        this.deviceTable.get(activeDevice).doClick(0);
    }    
    
    @Override
    public void actionPerformed(ActionEvent e)
    {
        int  i;       
        Vector<String> results;              

        if (deviceTable.containsKey(activeDevice))
        {
            deviceTable.get(activeDevice).setBackground(UIManager.getColor("Button.background"));
        }

        activeDevice = e.getActionCommand();
        activeIf = "";

        deviceTable.get(activeDevice).setBackground(Color.WHITE);

        nc.repaint();
        
        wc.removeAll();   
        
        cc.removeAll();
        cc.revalidate();
        cc.repaint();

        results = Main.db.getInterfaces(activeDevice);
        for (i = 0; i < results.size(); i++)
        {
            this.addInterface(results.get(i));
        }
             
        results = Main.db.getWireless(activeDevice);        
        for (i = 0; i < results.size(); i++)
        {
            this.addWireless(results.get(i));
        }
       
        this.addScanner("Scanned APs");                
    }
  
    class IfActionListener implements ActionListener
    {

        @Override
        public void actionPerformed(ActionEvent e)
        {
            Vector<String> results;

            if (ifTable.containsKey(activeIf))
            {
                ifTable.get(activeIf).setBackground(UIManager.getColor("Button.background"));
            }

            activeIf = e.getActionCommand();

            ifTable.get(activeIf).setBackground(Color.WHITE);

            wc.repaint();

            cc.removeAll();

            results = Main.db.getInterfaceData(activeDevice, activeIf);

            GridLayout gl = new GridLayout(10, 1, 0, 5);
            cc.setLayout(gl);

            addInterfaceData("     Mac Address:");
            addInterfaceData(results.get(2));
            addInterfaceData("     Ip Address:");
            addInterfaceData(results.get(3));
            addInterfaceData("     Subnet Mask:");
            addInterfaceData(results.get(4));
            addInterfaceData("     Network Address:");
            addInterfaceData(results.get(5));
            addInterfaceData("     Broadcast Address:");
            addInterfaceData(results.get(6));
            addInterfaceData("     Default Gateway:");
            addInterfaceData(results.get(7));
            addInterfaceData("     Maximum Bit Rate:");
            addInterfaceData(results.get(8));
            addInterfaceData("     Current Bit Rate:");
            addInterfaceData(results.get(9));
            addInterfaceData("     Current Used Range:");
            addInterfaceData(results.get(10));
            addInterfaceData("     Packet Error Rate:");
            addInterfaceData(results.get(11));

            cc.revalidate();
            cc.repaint();

        }
    }

    class WifActionListener implements ActionListener
    {

        @Override
        public void actionPerformed(ActionEvent e)
        {

            Vector<String> results;

            if (ifTable.containsKey(activeIf))
            {
                ifTable.get(activeIf).setBackground(UIManager.getColor("Button.background"));
            }

            activeIf = e.getActionCommand();

            ifTable.get(activeIf).setBackground(Color.WHITE);

            wc.repaint();

            cc.removeAll();

            results = Main.db.getWirelessData(activeDevice, activeIf);

            GridLayout gl = new GridLayout(21, 1, 0, 5);
            cc.setLayout(gl);

            addInterfaceData("     Mac Address:");
            addInterfaceData(results.get(2));
            addInterfaceData("     Ip Address:");
            addInterfaceData(results.get(3));
            addInterfaceData("     Subnet Mask:");
            addInterfaceData(results.get(4));
            addInterfaceData("     Network Address:");
            addInterfaceData(results.get(5));
            addInterfaceData("     Broadcast Address:");
            addInterfaceData(results.get(6));
            addInterfaceData("     Default Gateway:");
            addInterfaceData(results.get(7));
            addInterfaceData("     Maximum Bit Rate:");
            addInterfaceData(results.get(8));
            addInterfaceData("     Current Bit Rate:");
            addInterfaceData(results.get(9));
            addInterfaceData("     Current Used Range:");
            addInterfaceData(results.get(10));
            addInterfaceData("     Packet Error Rate:");
            addInterfaceData(results.get(11));
            addInterfaceData("     Access Point's Mac Address:");
            addInterfaceData(results.get(12));
            addInterfaceData("     Access Point's ESSID:");
            addInterfaceData(results.get(13));
            addInterfaceData("     Access Point's Channel:");
            addInterfaceData(results.get(14));
            addInterfaceData("     Access Point's Mode:");
            addInterfaceData(results.get(15));
            addInterfaceData("     Transmission Power:");
            addInterfaceData(results.get(16));
            addInterfaceData("     Link Quality:");
            addInterfaceData(results.get(17));
            addInterfaceData("     Signal Level:");
            addInterfaceData(results.get(18));
            addInterfaceData("     Noise Level:");
            addInterfaceData(results.get(19));
            addInterfaceData("     Discarded Packets:");
            addInterfaceData(results.get(20));

            cc.revalidate();
            cc.repaint();

        }
    }

    class ApActionListener implements ActionListener
    {

        @Override
        public void actionPerformed(ActionEvent e)
        {

            if (ifTable.containsKey(activeIf))
            {
                ifTable.get(activeIf).setBackground(UIManager.getColor("Button.background"));
            }

            activeIf = e.getActionCommand();

            ifTable.get(activeIf).setBackground(Color.WHITE);

            wc.repaint();

            cc.removeAll();

            cc.setLayout(new BoxLayout(cc, BoxLayout.Y_AXIS));

            String[] columns =
            {
                "Mac Address", "ESSID", "Channel", "Mode", "Signal Level"
            };

            addAccessPoint(null, columns);

            List<String[]> results = Main.db.getAccessPointsData(activeDevice);

            for (int i = 0; i < results.size(); i++)
            {
                addAccessPoint(results.get(i)[0], results.get(i));
            }

            cc.revalidate();
            cc.repaint();

        }
    }
    
    class TerminalActionListener implements ActionListener
    {

        @Override
        public void actionPerformed(ActionEvent e)
        {
            int  i;       
            Vector<String> results;              

            if (deviceTable.containsKey(activeDevice))
            {
                deviceTable.get(activeDevice).setBackground(UIManager.getColor("Button.background"));
            }

            activeDevice = e.getActionCommand();
            activeIf = "";

            deviceTable.get(activeDevice).setBackground(Color.WHITE);

            nc.repaint();

            wc.removeAll();   

            cc.removeAll();           

            results = Main.db.getTerinalData(activeDevice);

            GridLayout gl = new GridLayout(10, 1, 0, 5);
            cc.setLayout(gl);

            addInterfaceData("     IMEI:");
            addInterfaceData(activeDevice);
            addInterfaceData("     Model:");
            addInterfaceData(results.get(0));
            addInterfaceData("     Manufacturer:");
            addInterfaceData(results.get(1));
            addInterfaceData("     Version:");
            addInterfaceData(results.get(2));
            addInterfaceData("     Latitude:");
            addInterfaceData(results.get(3));
            addInterfaceData("     Longitude:");
            addInterfaceData(results.get(4));
            addInterfaceData("     Battery Level:");
            addInterfaceData(results.get(5));
            addInterfaceData("     Charging state:");
            addInterfaceData(results.get(6));          
            
            
            wc.revalidate();        
            wc.repaint();  
            cc.revalidate();
            cc.repaint();
        }
    }    
}
